# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .licensing_client import LicensingClient

__all__ = [
    'AccountEntitlement',
    'AccountEntitlementUpdateModel',
    'AccountLicenseExtensionUsage',
    'AccountLicenseUsage',
    'AccountRights',
    'AccountUserLicense',
    'ClientRightsContainer',
    'ExtensionAssignment',
    'ExtensionAssignmentDetails',
    'ExtensionLicenseData',
    'ExtensionOperationResult',
    'ExtensionRightsResult',
    'ExtensionSource',
    'GraphSubjectBase',
    'Identity',
    'IdentityBase',
    'IdentityMapping',
    'IdentityRef',
    'License',
    'MsdnEntitlement',
    'ReferenceLinks',
    'LicensingClient'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class CodeChangeTrendItem(Model):
    """CodeChangeTrendItem.

    :param time:
    :type time: datetime
    :param value:
    :type value: int
    """

    _attribute_map = {
        'time': {'key': 'time', 'type': 'iso-8601'},
        'value': {'key': 'value', 'type': 'int'}
    }

    def __init__(self, time=None, value=None):
        super(CodeChangeTrendItem, self).__init__()
        self.time = time
        self.value = value


class LanguageMetricsSecuredObject(Model):
    """LanguageMetricsSecuredObject.

    :param namespace_id:
    :type namespace_id: str
    :param project_id:
    :type project_id: str
    :param required_permissions:
    :type required_permissions: int
    """

    _attribute_map = {
        'namespace_id': {'key': 'namespaceId', 'type': 'str'},
        'project_id': {'key': 'projectId', 'type': 'str'},
        'required_permissions': {'key': 'requiredPermissions', 'type': 'int'}
    }

    def __init__(self, namespace_id=None, project_id=None, required_permissions=None):
        super(LanguageMetricsSecuredObject, self).__init__()
        self.namespace_id = namespace_id
        self.project_id = project_id
        self.required_permissions = required_permissions


class LanguageStatistics(LanguageMetricsSecuredObject):
    """LanguageStatistics.

    :param namespace_id:
    :type namespace_id: str
    :param project_id:
    :type project_id: str
    :param required_permissions:
    :type required_permissions: int
    :param bytes:
    :type bytes: long
    :param files:
    :type files: int
    :param files_percentage:
    :type files_percentage: float
    :param language_percentage:
    :type language_percentage: float
    :param name:
    :type name: str
    """

    _attribute_map = {
        'namespace_id': {'key': 'namespaceId', 'type': 'str'},
        'project_id': {'key': 'projectId', 'type': 'str'},
        'required_permissions': {'key': 'requiredPermissions', 'type': 'int'},
        'bytes': {'key': 'bytes', 'type': 'long'},
        'files': {'key': 'files', 'type': 'int'},
        'files_percentage': {'key': 'filesPercentage', 'type': 'float'},
        'language_percentage': {'key': 'languagePercentage', 'type': 'float'},
        'name': {'key': 'name', 'type': 'str'}
    }

    def __init__(self, namespace_id=None, project_id=None, required_permissions=None, bytes=None, files=None, files_percentage=None, language_percentage=None, name=None):
        super(LanguageStatistics, self).__init__(namespace_id=namespace_id, project_id=project_id, required_permissions=required_permissions)
        self.bytes = bytes
        self.files = files
        self.files_percentage = files_percentage
        self.language_percentage = language_percentage
        self.name = name


class ProjectActivityMetrics(Model):
    """ProjectActivityMetrics.

    :param authors_count:
    :type authors_count: int
    :param code_changes_count:
    :type code_changes_count: int
    :param code_changes_trend:
    :type code_changes_trend: list of :class:`CodeChangeTrendItem <azure.devops.v5_0.project_analysis.models.CodeChangeTrendItem>`
    :param project_id:
    :type project_id: str
    :param pull_requests_completed_count:
    :type pull_requests_completed_count: int
    :param pull_requests_created_count:
    :type pull_requests_created_count: int
    """

    _attribute_map = {
        'authors_count': {'key': 'authorsCount', 'type': 'int'},
        'code_changes_count': {'key': 'codeChangesCount', 'type': 'int'},
        'code_changes_trend': {'key': 'codeChangesTrend', 'type': '[CodeChangeTrendItem]'},
        'project_id': {'key': 'projectId', 'type': 'str'},
        'pull_requests_completed_count': {'key': 'pullRequestsCompletedCount', 'type': 'int'},
        'pull_requests_created_count': {'key': 'pullRequestsCreatedCount', 'type': 'int'}
    }

    def __init__(self, authors_count=None, code_changes_count=None, code_changes_trend=None, project_id=None, pull_requests_completed_count=None, pull_requests_created_count=None):
        super(ProjectActivityMetrics, self).__init__()
        self.authors_count = authors_count
        self.code_changes_count = code_changes_count
        self.code_changes_trend = code_changes_trend
        self.project_id = project_id
        self.pull_requests_completed_count = pull_requests_completed_count
        self.pull_requests_created_count = pull_requests_created_count


class ProjectLanguageAnalytics(LanguageMetricsSecuredObject):
    """ProjectLanguageAnalytics.

    :param namespace_id:
    :type namespace_id: str
    :param project_id:
    :type project_id: str
    :param required_permissions:
    :type required_permissions: int
    :param id:
    :type id: str
    :param language_breakdown:
    :type language_breakdown: list of :class:`LanguageStatistics <azure.devops.v5_0.project_analysis.models.LanguageStatistics>`
    :param repository_language_analytics:
    :type repository_language_analytics: list of :class:`RepositoryLanguageAnalytics <azure.devops.v5_0.project_analysis.models.RepositoryLanguageAnalytics>`
    :param result_phase:
    :type result_phase: object
    :param url:
    :type url: str
    """

    _attribute_map = {
        'namespace_id': {'key': 'namespaceId', 'type': 'str'},
        'project_id': {'key': 'projectId', 'type': 'str'},
        'required_permissions': {'key': 'requiredPermissions', 'type': 'int'},
        'id': {'key': 'id', 'type': 'str'},
        'language_breakdown': {'key': 'languageBreakdown', 'type': '[LanguageStatistics]'},
        'repository_language_analytics': {'key': 'repositoryLanguageAnalytics', 'type': '[RepositoryLanguageAnalytics]'},
        'result_phase': {'key': 'resultPhase', 'type': 'object'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, namespace_id=None, project_id=None, required_permissions=None, id=None, language_breakdown=None, repository_language_analytics=None, result_phase=None, url=None):
        super(ProjectLanguageAnalytics, self).__init__(namespace_id=namespace_id, project_id=project_id, required_permissions=required_permissions)
        self.id = id
        self.language_breakdown = language_breakdown
        self.repository_language_analytics = repository_language_analytics
        self.result_phase = result_phase
        self.url = url


class RepositoryActivityMetrics(Model):
    """RepositoryActivityMetrics.

    :param code_changes_count:
    :type code_changes_count: int
    :param code_changes_trend:
    :type code_changes_trend: list of :class:`CodeChangeTrendItem <azure.devops.v5_0.project_analysis.models.CodeChangeTrendItem>`
    :param repository_id:
    :type repository_id: str
    """

    _attribute_map = {
        'code_changes_count': {'key': 'codeChangesCount', 'type': 'int'},
        'code_changes_trend': {'key': 'codeChangesTrend', 'type': '[CodeChangeTrendItem]'},
        'repository_id': {'key': 'repositoryId', 'type': 'str'}
    }

    def __init__(self, code_changes_count=None, code_changes_trend=None, repository_id=None):
        super(RepositoryActivityMetrics, self).__init__()
        self.code_changes_count = code_changes_count
        self.code_changes_trend = code_changes_trend
        self.repository_id = repository_id


class RepositoryLanguageAnalytics(LanguageMetricsSecuredObject):
    """RepositoryLanguageAnalytics.

    :param namespace_id:
    :type namespace_id: str
    :param project_id:
    :type project_id: str
    :param required_permissions:
    :type required_permissions: int
    :param id:
    :type id: str
    :param language_breakdown:
    :type language_breakdown: list of :class:`LanguageStatistics <azure.devops.v5_0.project_analysis.models.LanguageStatistics>`
    :param name:
    :type name: str
    :param result_phase:
    :type result_phase: object
    :param updated_time:
    :type updated_time: datetime
    """

    _attribute_map = {
        'namespace_id': {'key': 'namespaceId', 'type': 'str'},
        'project_id': {'key': 'projectId', 'type': 'str'},
        'required_permissions': {'key': 'requiredPermissions', 'type': 'int'},
        'id': {'key': 'id', 'type': 'str'},
        'language_breakdown': {'key': 'languageBreakdown', 'type': '[LanguageStatistics]'},
        'name': {'key': 'name', 'type': 'str'},
        'result_phase': {'key': 'resultPhase', 'type': 'object'},
        'updated_time': {'key': 'updatedTime', 'type': 'iso-8601'}
    }

    def __init__(self, namespace_id=None, project_id=None, required_permissions=None, id=None, language_breakdown=None, name=None, result_phase=None, updated_time=None):
        super(RepositoryLanguageAnalytics, self).__init__(namespace_id=namespace_id, project_id=project_id, required_permissions=required_permissions)
        self.id = id
        self.language_breakdown = language_breakdown
        self.name = name
        self.result_phase = result_phase
        self.updated_time = updated_time


__all__ = [
    'CodeChangeTrendItem',
    'LanguageMetricsSecuredObject',
    'LanguageStatistics',
    'ProjectActivityMetrics',
    'ProjectLanguageAnalytics',
    'RepositoryActivityMetrics',
    'RepositoryLanguageAnalytics',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class SessionToken(Model):
    """
    :param access_id:
    :type access_id: str
    :param alternate_token: This is populated when user requests a compact token. The alternate token value is self describing token.
    :type alternate_token: str
    :param authorization_id:
    :type authorization_id: str
    :param claims:
    :type claims: dict
    :param client_id:
    :type client_id: str
    :param display_name:
    :type display_name: str
    :param host_authorization_id:
    :type host_authorization_id: str
    :param is_public:
    :type is_public: bool
    :param is_valid:
    :type is_valid: bool
    :param public_data:
    :type public_data: str
    :param scope:
    :type scope: str
    :param source:
    :type source: str
    :param target_accounts:
    :type target_accounts: list of str
    :param token: This is computed and not returned in Get queries
    :type token: str
    :param user_id:
    :type user_id: str
    :param valid_from:
    :type valid_from: datetime
    :param valid_to:
    :type valid_to: datetime
    """

    _attribute_map = {
        'access_id': {'key': 'accessId', 'type': 'str'},
        'alternate_token': {'key': 'alternateToken', 'type': 'str'},
        'authorization_id': {'key': 'authorizationId', 'type': 'str'},
        'claims': {'key': 'claims', 'type': '{str}'},
        'client_id': {'key': 'clientId', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'host_authorization_id': {'key': 'hostAuthorizationId', 'type': 'str'},
        'is_public': {'key': 'isPublic', 'type': 'bool'},
        'is_valid': {'key': 'isValid', 'type': 'bool'},
        'public_data': {'key': 'publicData', 'type': 'str'},
        'scope': {'key': 'scope', 'type': 'str'},
        'source': {'key': 'source', 'type': 'str'},
        'target_accounts': {'key': 'targetAccounts', 'type': '[str]'},
        'token': {'key': 'token', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'valid_from': {'key': 'validFrom', 'type': 'iso-8601'},
        'valid_to': {'key': 'validTo', 'type': 'iso-8601'}
    }

    def __init__(self, access_id=None, alternate_token=None, authorization_id=None, claims=None, client_id=None, display_name=None, host_authorization_id=None, is_public=None, is_valid=None, public_data=None, scope=None, source=None, target_accounts=None, token=None, user_id=None, valid_from=None, valid_to=None):
        super(SessionToken, self).__init__()
        self.access_id = access_id
        self.alternate_token = alternate_token
        self.authorization_id = authorization_id
        self.claims = claims
        self.client_id = client_id
        self.display_name = display_name
        self.host_authorization_id = host_authorization_id
        self.is_public = is_public
        self.is_valid = is_valid
        self.public_data = public_data
        self.scope = scope
        self.source = source
        self.target_accounts = target_accounts
        self.token = token
        self.user_id = user_id
        self.valid_from = valid_from
        self.valid_to = valid_to


class TokenAdminPagedSessionTokens(Model):
    """
    A paginated list of session tokens. Session tokens correspond to OAuth credentials such as personal access tokens (PATs) and other OAuth authorizations.

    :param continuation_token: The continuation token that can be used to retrieve the next page of session tokens, or <code>null</code> if there is no next page.
    :type continuation_token: str
    :param value: The list of all session tokens in the current page.
    :type value: list of :class:`SessionToken <azure.devops.v5_1.token_admin.models.SessionToken>`
    """

    _attribute_map = {
        'continuation_token': {'key': 'continuationToken', 'type': 'str'},
        'value': {'key': 'value', 'type': '[SessionToken]'}
    }

    def __init__(self, continuation_token=None, value=None):
        super(TokenAdminPagedSessionTokens, self).__init__()
        self.continuation_token = continuation_token
        self.value = value


class TokenAdminRevocation(Model):
    """
    A request to revoke a particular delegated authorization.

    :param authorization_id: The authorization ID of the OAuth authorization to revoke.
    :type authorization_id: str
    """

    _attribute_map = {
        'authorization_id': {'key': 'authorizationId', 'type': 'str'}
    }

    def __init__(self, authorization_id=None):
        super(TokenAdminRevocation, self).__init__()
        self.authorization_id = authorization_id


class TokenAdminRevocationRule(Model):
    """
    A rule which is applied to disable any incoming delegated authorization which matches the given properties.

    :param created_before: A datetime cutoff. Tokens created before this time will be rejected. This is an optional parameter. If omitted, defaults to the time at which the rule was created.
    :type created_before: datetime
    :param scopes: A string containing a space-delimited list of OAuth scopes. A token matching any one of the scopes will be rejected. For a list of all OAuth scopes supported by Azure DevOps, see: https://docs.microsoft.com/en-us/azure/devops/integrate/get-started/authentication/oauth?view=azure-devops#scopes This is a mandatory parameter.
    :type scopes: str
    """

    _attribute_map = {
        'created_before': {'key': 'createdBefore', 'type': 'iso-8601'},
        'scopes': {'key': 'scopes', 'type': 'str'}
    }

    def __init__(self, created_before=None, scopes=None):
        super(TokenAdminRevocationRule, self).__init__()
        self.created_before = created_before
        self.scopes = scopes


__all__ = [
    'SessionToken',
    'TokenAdminPagedSessionTokens',
    'TokenAdminRevocation',
    'TokenAdminRevocationRule',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger

logger = get_logger(__name__)


def trim_for_display(text, max_length):
    if not text:
        return text

    result = text
    if len(text) > max_length:
        result = text[0: max_length] + '...'
        logger.info('trimmed %s to %s', text, result)
    else:
        logger.info('not trimming %s', text)

    return result


def date_time_to_only_date(date_time):
    try:
        from dateutil import parser
        result = str(parser.parse(date_time).date())
        logger.info('trimmed %s to %s', date_time, result)
        return result
    except Exception as ex:  # pylint: disable=broad-except
        logger.info('encountered error while parsing date time')
        logger.info(str(ex))
        return date_time



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class GraphSubjectBase(Model):
    """GraphSubjectBase.

    :param _links: This field contains zero or more interesting links about the graph subject. These links may be invoked to obtain additional relationships or more detailed information about this graph subject.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.microsoft._visual_studio._services._web_api.models.ReferenceLinks>`
    :param descriptor: The descriptor is the primary way to reference the graph subject while the system is running. This field will uniquely identify the same graph subject across both Accounts and Organizations.
    :type descriptor: str
    :param display_name: This is the non-unique display name of the graph subject. To change this field, you must alter its value in the source provider.
    :type display_name: str
    :param url: This url is the full route to the source resource of this graph subject.
    :type url: str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, _links=None, descriptor=None, display_name=None, url=None):
        super(GraphSubjectBase, self).__init__()
        self._links = _links
        self.descriptor = descriptor
        self.display_name = display_name
        self.url = url


class IdentityRef(GraphSubjectBase):
    """IdentityRef.

    :param _links: This field contains zero or more interesting links about the graph subject. These links may be invoked to obtain additional relationships or more detailed information about this graph subject.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.microsoft._visual_studio._services._web_api.models.ReferenceLinks>`
    :param descriptor: The descriptor is the primary way to reference the graph subject while the system is running. This field will uniquely identify the same graph subject across both Accounts and Organizations.
    :type descriptor: str
    :param display_name: This is the non-unique display name of the graph subject. To change this field, you must alter its value in the source provider.
    :type display_name: str
    :param url: This url is the full route to the source resource of this graph subject.
    :type url: str
    :param directory_alias:
    :type directory_alias: str
    :param id:
    :type id: str
    :param image_url:
    :type image_url: str
    :param inactive:
    :type inactive: bool
    :param is_aad_identity:
    :type is_aad_identity: bool
    :param is_container:
    :type is_container: bool
    :param is_deleted_in_origin:
    :type is_deleted_in_origin: bool
    :param profile_url:
    :type profile_url: str
    :param unique_name:
    :type unique_name: str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'descriptor': {'key': 'descriptor', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        'directory_alias': {'key': 'directoryAlias', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'image_url': {'key': 'imageUrl', 'type': 'str'},
        'inactive': {'key': 'inactive', 'type': 'bool'},
        'is_aad_identity': {'key': 'isAadIdentity', 'type': 'bool'},
        'is_container': {'key': 'isContainer', 'type': 'bool'},
        'is_deleted_in_origin': {'key': 'isDeletedInOrigin', 'type': 'bool'},
        'profile_url': {'key': 'profileUrl', 'type': 'str'},
        'unique_name': {'key': 'uniqueName', 'type': 'str'}
    }

    def __init__(self, _links=None, descriptor=None, display_name=None, url=None, directory_alias=None, id=None, image_url=None, inactive=None, is_aad_identity=None, is_container=None, is_deleted_in_origin=None, profile_url=None, unique_name=None):
        super(IdentityRef, self).__init__(_links=_links, descriptor=descriptor, display_name=display_name, url=url)
        self.directory_alias = directory_alias
        self.id = id
        self.image_url = image_url
        self.inactive = inactive
        self.is_aad_identity = is_aad_identity
        self.is_container = is_container
        self.is_deleted_in_origin = is_deleted_in_origin
        self.profile_url = profile_url
        self.unique_name = unique_name


class PolicyConfigurationRef(Model):
    """PolicyConfigurationRef.

    :param id: The policy configuration ID.
    :type id: int
    :param type: The policy configuration type.
    :type type: :class:`PolicyTypeRef <azure.devops.v5_0.policy.models.PolicyTypeRef>`
    :param url: The URL where the policy configuration can be retrieved.
    :type url: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'int'},
        'type': {'key': 'type', 'type': 'PolicyTypeRef'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, id=None, type=None, url=None):
        super(PolicyConfigurationRef, self).__init__()
        self.id = id
        self.type = type
        self.url = url


class PolicyEvaluationRecord(Model):
    """PolicyEvaluationRecord.

    :param _links: Links to other related objects
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.policy.models.ReferenceLinks>`
    :param artifact_id: A string which uniquely identifies the target of a policy evaluation.
    :type artifact_id: str
    :param completed_date: Time when this policy finished evaluating on this pull request.
    :type completed_date: datetime
    :param configuration: Contains all configuration data for the policy which is being evaluated.
    :type configuration: :class:`PolicyConfiguration <azure.devops.v5_0.policy.models.PolicyConfiguration>`
    :param context: Internal context data of this policy evaluation.
    :type context: :class:`object <azure.devops.v5_0.policy.models.object>`
    :param evaluation_id: Guid which uniquely identifies this evaluation record (one policy running on one pull request).
    :type evaluation_id: str
    :param started_date: Time when this policy was first evaluated on this pull request.
    :type started_date: datetime
    :param status: Status of the policy (Running, Approved, Failed, etc.)
    :type status: object
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'artifact_id': {'key': 'artifactId', 'type': 'str'},
        'completed_date': {'key': 'completedDate', 'type': 'iso-8601'},
        'configuration': {'key': 'configuration', 'type': 'PolicyConfiguration'},
        'context': {'key': 'context', 'type': 'object'},
        'evaluation_id': {'key': 'evaluationId', 'type': 'str'},
        'started_date': {'key': 'startedDate', 'type': 'iso-8601'},
        'status': {'key': 'status', 'type': 'object'}
    }

    def __init__(self, _links=None, artifact_id=None, completed_date=None, configuration=None, context=None, evaluation_id=None, started_date=None, status=None):
        super(PolicyEvaluationRecord, self).__init__()
        self._links = _links
        self.artifact_id = artifact_id
        self.completed_date = completed_date
        self.configuration = configuration
        self.context = context
        self.evaluation_id = evaluation_id
        self.started_date = started_date
        self.status = status


class PolicyTypeRef(Model):
    """PolicyTypeRef.

    :param display_name: Display name of the policy type.
    :type display_name: str
    :param id: The policy type ID.
    :type id: str
    :param url: The URL where the policy type can be retrieved.
    :type url: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, display_name=None, id=None, url=None):
        super(PolicyTypeRef, self).__init__()
        self.display_name = display_name
        self.id = id
        self.url = url


class ReferenceLinks(Model):
    """ReferenceLinks.

    :param links: The readonly view of the links.  Because Reference links are readonly, we only want to expose them as read only.
    :type links: dict
    """

    _attribute_map = {
        'links': {'key': 'links', 'type': '{object}'}
    }

    def __init__(self, links=None):
        super(ReferenceLinks, self).__init__()
        self.links = links


class VersionedPolicyConfigurationRef(PolicyConfigurationRef):
    """VersionedPolicyConfigurationRef.

    :param id: The policy configuration ID.
    :type id: int
    :param type: The policy configuration type.
    :type type: :class:`PolicyTypeRef <azure.devops.v5_0.policy.models.PolicyTypeRef>`
    :param url: The URL where the policy configuration can be retrieved.
    :type url: str
    :param revision: The policy configuration revision ID.
    :type revision: int
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'int'},
        'type': {'key': 'type', 'type': 'PolicyTypeRef'},
        'url': {'key': 'url', 'type': 'str'},
        'revision': {'key': 'revision', 'type': 'int'}
    }

    def __init__(self, id=None, type=None, url=None, revision=None):
        super(VersionedPolicyConfigurationRef, self).__init__(id=id, type=type, url=url)
        self.revision = revision


class PolicyConfiguration(VersionedPolicyConfigurationRef):
    """PolicyConfiguration.

    :param id: The policy configuration ID.
    :type id: int
    :param type: The policy configuration type.
    :type type: :class:`PolicyTypeRef <azure.devops.v5_0.policy.models.PolicyTypeRef>`
    :param url: The URL where the policy configuration can be retrieved.
    :type url: str
    :param revision: The policy configuration revision ID.
    :type revision: int
    :param _links: The links to other objects related to this object.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.policy.models.ReferenceLinks>`
    :param created_by: A reference to the identity that created the policy.
    :type created_by: :class:`IdentityRef <azure.devops.v5_0.policy.models.IdentityRef>`
    :param created_date: The date and time when the policy was created.
    :type created_date: datetime
    :param is_blocking: Indicates whether the policy is blocking.
    :type is_blocking: bool
    :param is_deleted: Indicates whether the policy has been (soft) deleted.
    :type is_deleted: bool
    :param is_enabled: Indicates whether the policy is enabled.
    :type is_enabled: bool
    :param settings: The policy configuration settings.
    :type settings: :class:`object <azure.devops.v5_0.policy.models.object>`
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'int'},
        'type': {'key': 'type', 'type': 'PolicyTypeRef'},
        'url': {'key': 'url', 'type': 'str'},
        'revision': {'key': 'revision', 'type': 'int'},
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'created_by': {'key': 'createdBy', 'type': 'IdentityRef'},
        'created_date': {'key': 'createdDate', 'type': 'iso-8601'},
        'is_blocking': {'key': 'isBlocking', 'type': 'bool'},
        'is_deleted': {'key': 'isDeleted', 'type': 'bool'},
        'is_enabled': {'key': 'isEnabled', 'type': 'bool'},
        'settings': {'key': 'settings', 'type': 'object'}
    }

    def __init__(self, id=None, type=None, url=None, revision=None, _links=None, created_by=None, created_date=None, is_blocking=None, is_deleted=None, is_enabled=None, settings=None):
        super(PolicyConfiguration, self).__init__(id=id, type=type, url=url, revision=revision)
        self._links = _links
        self.created_by = created_by
        self.created_date = created_date
        self.is_blocking = is_blocking
        self.is_deleted = is_deleted
        self.is_enabled = is_enabled
        self.settings = settings


class PolicyType(PolicyTypeRef):
    """PolicyType.

    :param display_name: Display name of the policy type.
    :type display_name: str
    :param id: The policy type ID.
    :type id: str
    :param url: The URL where the policy type can be retrieved.
    :type url: str
    :param _links: The links to other objects related to this object.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.policy.models.ReferenceLinks>`
    :param description: Detailed description of the policy type.
    :type description: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'description': {'key': 'description', 'type': 'str'}
    }

    def __init__(self, display_name=None, id=None, url=None, _links=None, description=None):
        super(PolicyType, self).__init__(display_name=display_name, id=id, url=url)
        self._links = _links
        self.description = description


__all__ = [
    'GraphSubjectBase',
    'IdentityRef',
    'PolicyConfigurationRef',
    'PolicyEvaluationRecord',
    'PolicyTypeRef',
    'ReferenceLinks',
    'VersionedPolicyConfigurationRef',
    'PolicyConfiguration',
    'PolicyType',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class UPackLimitedPackageMetadata(Model):
    """
    :param version:
    :type version: str
    """

    _attribute_map = {
        'version': {'key': 'version', 'type': 'str'}
    }

    def __init__(self, version=None):
        super(UPackLimitedPackageMetadata, self).__init__()
        self.version = version


class UPackLimitedPackageMetadataListResponse(Model):
    """
    :param count:
    :type count: int
    :param value:
    :type value: list of :class:`UPackLimitedPackageMetadata <azure.devops.v5_1.upack.models.UPackLimitedPackageMetadata>`
    """

    _attribute_map = {
        'count': {'key': 'count', 'type': 'int'},
        'value': {'key': 'value', 'type': '[UPackLimitedPackageMetadata]'}
    }

    def __init__(self, count=None, value=None):
        super(UPackLimitedPackageMetadataListResponse, self).__init__()
        self.count = count
        self.value = value


class UPackPackageMetadata(Model):
    """
    :param description:
    :type description: str
    :param manifest_id:
    :type manifest_id: str
    :param super_root_id:
    :type super_root_id: str
    :param version:
    :type version: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'manifest_id': {'key': 'manifestId', 'type': 'str'},
        'super_root_id': {'key': 'superRootId', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'}
    }

    def __init__(self, description=None, manifest_id=None, super_root_id=None, version=None):
        super(UPackPackageMetadata, self).__init__()
        self.description = description
        self.manifest_id = manifest_id
        self.super_root_id = super_root_id
        self.version = version


class UPackPackagePushMetadata(UPackPackageMetadata):
    """
    :param description:
    :type description: str
    :param manifest_id:
    :type manifest_id: str
    :param super_root_id:
    :type super_root_id: str
    :param version:
    :type version: str
    :param proof_nodes:
    :type proof_nodes: list of str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'manifest_id': {'key': 'manifestId', 'type': 'str'},
        'super_root_id': {'key': 'superRootId', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'proof_nodes': {'key': 'proofNodes', 'type': '[str]'}
    }

    def __init__(self, description=None, manifest_id=None, super_root_id=None, version=None, proof_nodes=None):
        super(UPackPackagePushMetadata, self).__init__(description=description, manifest_id=manifest_id, super_root_id=super_root_id, version=version)
        self.proof_nodes = proof_nodes


class UPackPackageVersionDeletionState(Model):
    """
    Deletion state of a Universal package.

    :param deleted_date: UTC date the package was deleted.
    :type deleted_date: datetime
    :param name: Name of the package.
    :type name: str
    :param version: Version of the package.
    :type version: str
    """

    _attribute_map = {
        'deleted_date': {'key': 'deletedDate', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'}
    }

    def __init__(self, deleted_date=None, name=None, version=None):
        super(UPackPackageVersionDeletionState, self).__init__()
        self.deleted_date = deleted_date
        self.name = name
        self.version = version


__all__ = [
    'UPackLimitedPackageMetadata',
    'UPackLimitedPackageMetadataListResponse',
    'UPackPackageMetadata',
    'UPackPackagePushMetadata',
    'UPackPackageVersionDeletionState',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps


def load_team_help():
    helps['devops'] = """
    type: group
    short-summary: Manage Azure DevOps organization level operations.
    long-summary: |
        Related Groups
        az pipelines: Manage Azure Pipelines
        az boards: Manage Azure Boards
        az repos: Manage Azure Repos
        az artifacts: Manage Azure Artifacts
        """

    helps['devops project'] = """
    type: group
    short-summary: Manage team projects.
    """

    helps['devops service-endpoint'] = """
    type: group
    short-summary: Manage service endpoints/connections
    """

    helps['devops service-endpoint create'] = """
    type: command
    short-summary: Create a service endpoint using configuration file.
    long-summary: You can learn more about this at https://aka.ms/azure-devops-service-endpoint-config
    """

    helps['devops service-endpoint azurerm'] = """
    type: group
    short-summary: Manage Azure RM service endpoints/connections
    """

    helps['devops service-endpoint azurerm create'] = """
    type: command
    short-summary: Create an Azure RM type service endpoint.
    long-summary: For automation, set service principal password/secret in
      AZURE_DEVOPS_EXT_AZURE_RM_SERVICE_PRINCIPAL_KEY environment variable. You can learn
      more about this at https://aka.ms/azure-devops-cli-azurerm-service-endpoint
    """

    helps['devops service-endpoint github'] = """
    type: group
    short-summary: Manage GitHub service endpoints/connections
    """

    helps['devops service-endpoint github create'] = """
    type: command
    short-summary: Create a GitHub service endpoint.
    long-summary: For automation, set GitHub PAT token in AZURE_DEVOPS_EXT_GITHUB_PAT
      environment variable. You can learn more about this at https://aka.ms/azure-devops-cli-service-endpoint
    """

    helps['devops security'] = """
    type: group
    short-summary: Manage security related operations
    """

    helps['devops security group'] = """
    type: group
    short-summary: Manage security groups
    """

    helps['devops security group create'] = """
    type: command
    short-summary: Create a new Azure DevOps group.
    long-summary:
    examples:
          - name: Create an Azure DevOps Group with name and description
            text: |
              az devops security group create --name 'Some group name'
              --description 'Something to describe this group'
          - name: Add an existing AAD group to an Azure DevOps group
            text: |
              Get object ID of an existing AAD group
              az ad group show -g '{Group Name}'
              az devops security group create --origin-id '{Object ID}' --groups 'vssgp.someDescriptorForGroup'
          - name: Add an existing AAD group to an Azure DevOps group with AAD group Email ID
            text: |
              az devops security group create --email-id '{Email ID of AAD group}'
              --groups 'vssgp.someDescriptorForGroup'
          - name: Create a new Azure DevOps group and add it to existing Azure DevOps groups.
            text: |
              az devops security group create --name 'Some group name'
              --groups 'vssgp.someDescriptorForGroupOne,vssgp.someDescriptorForGroupTwo'
    """

    helps['devops security group membership'] = """
    type: group
    short-summary: Manage memberships for security groups
    """

    helps['devops security permission'] = """
    type: group
    short-summary: Manage security permissions
    """

    helps['devops security permission namespace'] = """
    type: group
    short-summary: Manage security namespaces
    """

    helps['devops security permission update'] = """
    type: command
    short-summary: Assign allow or deny permission to given user/group.
    long-summary: Learn more at https://aka.ms/azure-devops-cli-security-permission
    examples:
        - name: Assign view, edit and delete permission for team projects.
          text: |
            az devops security permission update  --allow-bit 7 --namespace-id\
            52d39943-cb85-4d7f-8fa8-c6baac873819 --subject user@fabrikam.com\
            --token "`$PROJECT:vstfs:///Classification/TeamProject/e479xxxx-2be8-xxxx-bb0b-3a0209cxxxx"

            You would need to add the bit value of the various permission bits to
            simultaneously allow/deny multiple permissions.

    """

    helps['devops team'] = """
    type: group
    short-summary: Manage teams
    """

    helps['devops user'] = """
    type: group
    short-summary: Manage users
    """

    helps['devops extension'] = """
    type: group
    short-summary: Manage extensions
    """

    helps['devops invoke'] = """
    type: command
    short-summary: This command will invoke request for any DevOps area and resource.
                   Please use only json output as the response of this command is not fixed.
                   Helpful docs -
                   https://docs.microsoft.com/en-us/rest/api/azure/devops/
    long-summary:
    examples:
        - name: Discover areas related to 'Wiki'
          text: |
            az devops invoke --query "[?contains(area,'wiki')]"

        - name: Get all wikis in a project
          text: |
            az devops invoke --area wiki --resource wikis --route-parameters project={Project Name} -o json
            az devops invoke --area wiki --resource wikis --route-parameters project=WikiIssue -o json

        - name: Add page to a wiki
          text: |
            az devops invoke --area wiki --resource pages --route-parameters project={Project Name}
            wikiIdentifier={Wiki Id} --in-file {Full File Path containing text to add to wiki}
            --query-parameters path={Page Path In Wiki} --http-method PUT --api-version 5.1-preview -o json
            az devops invoke --area wiki --resource pages --route-parameters project=WikiIssue
            wikiIdentifier=e479f402-2be8-4953-bb0b-3a0209cbc2d2 --in-file D:/temp/createWikiRequestBody.txt
            --encoding 'utf-8' --query-parameters path=sample738 --http-method PUT --api-version 5.1-preview -o json
    """

    helps['devops wiki'] = """
    type: group
    short-summary: Manage wikis
    """

    helps['devops wiki create'] = """
    type: command
    examples:
        - name: Create a project wiki
          text: |
            az devops wiki create --name myprojectwiki

        - name: Create a code wiki from a folder in a code repository
          text: |
            az devops wiki create --name WIKI_NAME --type codewiki --version BRANCH_NAME
            --repository REPO_NAME --mapped-path PATH_TO_PUBLISH
    """

    helps['devops wiki list'] = """
    type: command
    examples:
        - name: List all wikis for a project
          text: |
            az devops wiki list

        - name: List all wikis in the organization
          text: |
            az devops wiki list --scope organization
    """

    helps['devops wiki page'] = """
    type: group
    short-summary: Manage wiki pages
    """

    helps['devops wiki page create'] = """
    type: command
    examples:
        - name: Create a new page with path 'my page' in a wiki named 'myprojectwiki' with inline content
          text: |
            az devops wiki page create --path 'my page' --wiki myprojectwiki --content "Hello World"

        - name: Create a new page with path 'my page' in a wiki named 'myprojectwiki' with content from a file
          text: |
            az devops wiki page create --path 'my page' --wiki myprojectwiki --file-path a.txt\
            --encoding utf-8
    """

    helps['devops wiki page update'] = """
    type: command
    examples:
        - name: Update content of page with path 'my page' in a wiki named 'myprojectwiki' with inline content
          text: |
            az devops wiki page update --path 'my page' --wiki myprojectwiki --content "Hello World"\
            --version 4ae78ad5835cb7dd55072fe210c9ee7eb6d6413b

        - name: Update content of page with path 'my page' in a wiki with content from a file
          text: |
            az devops wiki page update --path 'my page' --wiki myprojectwiki --file-path a.txt\
            --encoding utf-8 --version 4ae78ad5835cb7dd55072fe210c9ee7eb6d6413b
    """



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

try:
    # Attempt to load mock (works on Python 3.3 and above)
    from unittest.mock import patch
except ImportError:
    # Attempt to load mock (works on Python version below 3.3)
    from mock import patch

from knack.util import CLIError
from knack.prompting import NoTTYException
from azext_devops.dev.team.service_endpoint import (list_service_endpoints,
                                                    show_service_endpoint,
                                                    create_service_endpoint,
                                                    create_github_service_endpoint,
                                                    create_azurerm_service_endpoint,
                                                    delete_service_endpoint,
                                                    update_service_endpoint)

from azext_devops.dev.common.services import clear_connection_cache
from azext_devops.test.utils.authentication import AuthenticatedTests
from azext_devops.test.utils.helper import get_client_mock_helper


class TestServiceEndpointMethods(AuthenticatedTests):

    _TEST_DEVOPS_ORGANIZATION = 'https://dev.azure.com/someorg'
    _TEST_PROJECT_NAME = 'sample_project'

    def setUp(self):
        self.authentication_setup()
        self.authenticate()
        self.get_client = patch('azext_devops.devops_sdk.connection.Connection.get_client', new=get_client_mock_helper)
        self.get_SEs_patcher = patch('azext_devops.devops_sdk.v5_0.service_endpoint.service_endpoint_client.ServiceEndpointClient.get_service_endpoints')
        self.get_SE_details_patcher = patch('azext_devops.devops_sdk.v5_0.service_endpoint.service_endpoint_client.ServiceEndpointClient.get_service_endpoint_details')
        self.create_SE_patcher = patch('azext_devops.devops_sdk.v5_0.service_endpoint.service_endpoint_client.ServiceEndpointClient.create_service_endpoint')
        self.delete_SE_patcher = patch('azext_devops.devops_sdk.v5_0.service_endpoint.service_endpoint_client.ServiceEndpointClient.delete_service_endpoint')
        self.set_authorize_endpoint = patch('azext_devops.dev.pipelines.pipeline_utils.set_authorize_resource')
        self.get_authorize_endpoint = patch('azext_devops.dev.pipelines.pipeline_utils.get_authorize_resource')

        self.mock_get_client = self.get_client.start()
        self.mock_get_SEs = self.get_SEs_patcher.start()
        self.mock_get_SE_detail = self.get_SE_details_patcher.start()
        self.mock_create_SE = self.create_SE_patcher.start()
        self.mock_delete_SE = self.delete_SE_patcher.start()
        self.mock_set_authorize = self.set_authorize_endpoint.start()
        self.mock_get_authorize = self.get_authorize_endpoint.start()

        #clear connection cache before running each test
        clear_connection_cache()

    def tearDown(self):
        patch.stopall()

    def test_list_service_endpoint(self):
        response = list_service_endpoints(self._TEST_DEVOPS_ORGANIZATION, self._TEST_PROJECT_NAME)
        
        #assert
        self.mock_get_SEs.assert_called_once_with(self._TEST_PROJECT_NAME)

    def test_get_service_endpoint(self):
        randomId = 'abcdfe34343'
        response = show_service_endpoint(randomId, self._TEST_DEVOPS_ORGANIZATION, self._TEST_PROJECT_NAME)

        #assert
        self.mock_get_SE_detail.assert_called_once_with(self._TEST_PROJECT_NAME, randomId)

    def test_delete_service_endpoint(self):
        randomId = 'abcdfe34343'
        delete_service_endpoint(randomId, 'false', self._TEST_DEVOPS_ORGANIZATION, self._TEST_PROJECT_NAME)

        #assert 
        self.mock_delete_SE.assert_called_once_with(self._TEST_PROJECT_NAME, randomId, 'false')

    def test_update_service_endpoint(self):
        randomId = 'abcdfe34343'
        update_service_endpoint(id=randomId, enable_for_all=True, organization=self._TEST_DEVOPS_ORGANIZATION,
                                project=self._TEST_PROJECT_NAME)

        #assert 
        self.mock_get_SE_detail.assert_called_once_with(self._TEST_PROJECT_NAME, randomId)
        self.mock_set_authorize.assert_called_once()
        self.mock_get_authorize.assert_called_once()


    def test_update_without_params_service_endpoint(self):
        randomId = 'abcdfe34343'
        try:
            update_service_endpoint(id=randomId, enable_for_all=None, organization=self._TEST_DEVOPS_ORGANIZATION,
                                    project=self._TEST_PROJECT_NAME)
        except CLIError as ex:
            self.assertEqual(str(ex), 'Atleast one property to be updated must be specified.')
            self.mock_get_SE_detail.assert_not_called()
            self.mock_set_authorize.assert_not_called()
            self.mock_get_authorize.assert_not_called()

    def test_create_service_endpoint_github(self):
        import os
        os.environ['AZURE_DEVOPS_EXT_GITHUB_PAT'] = 'fakeToken'
        response = create_github_service_endpoint(name = '', github_url='',
                                                  organization = self._TEST_DEVOPS_ORGANIZATION, 
                                                  project = self._TEST_PROJECT_NAME)
        del os.environ['AZURE_DEVOPS_EXT_GITHUB_PAT']

        #assert
        # not doing extensive comparision because object creation code does not have much logic
        self.mock_create_SE.assert_called_once()

    def test_create_service_endpoint_azure_rm(self):
        import os
        os.environ['AZURE_DEVOPS_EXT_AZURE_RM_SERVICE_PRINCIPAL_KEY'] = 'fakeKey'
        response = create_azurerm_service_endpoint(name = '',
                                                   azure_rm_tenant_id='',
                                                   azure_rm_service_principal_id='', 
                                                   azure_rm_subscription_id='',
                                                   azure_rm_subscription_name='',
                                                   organization = self._TEST_DEVOPS_ORGANIZATION,
                                                   project = self._TEST_PROJECT_NAME)
        del os.environ['AZURE_DEVOPS_EXT_AZURE_RM_SERVICE_PRINCIPAL_KEY']

        #assert
        # not doing extensive comparision because object creation code does not have much logic
        self.mock_create_SE.assert_called_once()

    def test_create_service_endpoint_ttyi_exception_github(self):
        try:
            response = create_github_service_endpoint(name = '', github_url='',
                                                      organization = self._TEST_DEVOPS_ORGANIZATION,
                                                      project = self._TEST_PROJECT_NAME)
            self.fail('exception was expected')
        except NoTTYException as ex:
            self.assertEqual(str(ex), 'Please pass GitHub access token in AZURE_DEVOPS_EXT_GITHUB_PAT environment variable in non-interactive mode.')

    def test_create_service_endpoint_ttyi_exception_azure_se(self):
        try:
            response = create_azurerm_service_endpoint(name = '',
                                                       azure_rm_tenant_id='',
                                                       azure_rm_service_principal_id='', 
                                                       azure_rm_subscription_id='',
                                                       azure_rm_subscription_name='',
                                                       organization = self._TEST_DEVOPS_ORGANIZATION,
                                                       project = self._TEST_PROJECT_NAME)
            self.fail('exception was expected')
        except NoTTYException as ex:
            self.assertEqual(str(ex), 'Please specify azure service principal key in AZURE_DEVOPS_EXT_AZURE_RM_SERVICE_PRINCIPAL_KEY environment variable in non-interactive mode or use --azure-rm-service-principal-certificate-path.')

if __name__ == '__main__':
    unittest.main()


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

try:
    # Attempt to load mock (works on Python 3.3 and above)
    from unittest.mock import patch
except ImportError:
    # Attempt to load mock (works on Python version below 3.3)
    from mock import patch

from azext_devops.dev.artifacts.universal import (publish_package,
                                                 download_package)
from azext_devops.dev.common.artifacttool import ArtifactToolInvoker
from azext_devops.dev.common.const import ARTIFACTTOOL_PAT_ENVKEY
from azext_devops.dev.common.services import clear_connection_cache
from knack.util import CLIError



class TestUniversalPackages(unittest.TestCase):

    _CONST_ORGANIZATION_SCOPE = 'organization'
    _CONST_PROJECT_SCOPE = 'project'
    _TEST_DEVOPS_ORGANIZATION = 'https://someorg.visualstudio.com'
    _TEST_DEVOPS_PROJECT = 'my-test-project'
    _TEST_PAT_TOKEN = 'somepattoken'
    _TEST_FEED_NAME = 'my-test-feed'
    _TEST_PACKAGE_NAME = 'my-test-package'
    _TEST_PACKAGE_VERSION = '0.0.1-preview'
    _TEST_PACKAGE_DESCRIPTION = 'test description'
    _TEST_PATH = '.'
    _TEST_FILTER = '*'

    def setUp(self):

        self.run_artifacttool = patch('azext_devops.dev.common.artifacttool.ArtifactToolInvoker.run_artifacttool')

        #start the patchers
        self.mock_run_artifacttool = self.run_artifacttool.start()
        
        #clear connection cache before running each test
        clear_connection_cache()


    def tearDown(self):
        patch.stopall()


    def test_publish_package(self):
        response = publish_package(feed = self._TEST_FEED_NAME,
            name = self._TEST_PACKAGE_NAME,
            version = self._TEST_PACKAGE_VERSION,
            description = self._TEST_PACKAGE_DESCRIPTION,
            path = self._TEST_PATH,
            organization = self._TEST_DEVOPS_ORGANIZATION)

        # assert
        self.mock_run_artifacttool.assert_called_with(self._TEST_DEVOPS_ORGANIZATION,
            [
                'universal', 'publish',
                '--service', self._TEST_DEVOPS_ORGANIZATION,
                 '--patvar', ARTIFACTTOOL_PAT_ENVKEY,
                 '--feed', self._TEST_FEED_NAME,
                 '--package-name', self._TEST_PACKAGE_NAME,
                 '--package-version', self._TEST_PACKAGE_VERSION,
                 '--path', self._TEST_PATH,
                 '--description', self._TEST_PACKAGE_DESCRIPTION
            ],
            'Publishing')

    def test_publish_package_with_project(self):
        response = publish_package(feed = self._TEST_FEED_NAME,
            name = self._TEST_PACKAGE_NAME,
            version = self._TEST_PACKAGE_VERSION,
            description = self._TEST_PACKAGE_DESCRIPTION,
            path = self._TEST_PATH,
            organization = self._TEST_DEVOPS_ORGANIZATION,
            project = self._TEST_DEVOPS_PROJECT,
            scope = self._CONST_PROJECT_SCOPE)

        # assert
        self.mock_run_artifacttool.assert_called_with(self._TEST_DEVOPS_ORGANIZATION,
            [
                'universal', 'publish',
                '--service', self._TEST_DEVOPS_ORGANIZATION,
                 '--patvar', ARTIFACTTOOL_PAT_ENVKEY,
                 '--feed', self._TEST_FEED_NAME,
                 '--package-name', self._TEST_PACKAGE_NAME,
                 '--package-version', self._TEST_PACKAGE_VERSION,
                 '--path', self._TEST_PATH,
                 '--project', self._TEST_DEVOPS_PROJECT,
                 '--description', self._TEST_PACKAGE_DESCRIPTION,
            ],
            'Publishing')

    def test_publish_package_when_project_without_scope_then_exception(self):
        with self.assertRaises(CLIError) as exc:
            response = publish_package(feed = self._TEST_FEED_NAME,
                name = self._TEST_PACKAGE_NAME,
                version = self._TEST_PACKAGE_VERSION,
                description = self._TEST_PACKAGE_DESCRIPTION,
                path = self._TEST_PATH,
                organization = self._TEST_DEVOPS_ORGANIZATION,
                project = self._TEST_DEVOPS_PROJECT)
        self.assertIn('--scope \'project\' is required when specifying a value in --project', str(exc.exception))
        
    def test_download_package(self):
        response = download_package(feed = self._TEST_FEED_NAME,
            name = self._TEST_PACKAGE_NAME,
            version = self._TEST_PACKAGE_VERSION,
            path = self._TEST_PATH,
            file_filter= self._TEST_FILTER,
            organization = self._TEST_DEVOPS_ORGANIZATION)

        # assert
        self.mock_run_artifacttool.assert_called_with(self._TEST_DEVOPS_ORGANIZATION,
            [
                'universal', 'download',
                '--service', self._TEST_DEVOPS_ORGANIZATION,
                    '--patvar', ARTIFACTTOOL_PAT_ENVKEY,
                    '--feed', self._TEST_FEED_NAME,
                    '--package-name', self._TEST_PACKAGE_NAME,
                    '--package-version', self._TEST_PACKAGE_VERSION,
                    '--path', self._TEST_PATH,
                    '--filter', self._TEST_FILTER,
            ],
            'Downloading')

    def test_download_package_with_project(self):
        response = download_package(feed = self._TEST_FEED_NAME,
            name = self._TEST_PACKAGE_NAME,
            version = self._TEST_PACKAGE_VERSION,
            path = self._TEST_PATH,
            file_filter= self._TEST_FILTER,
            organization = self._TEST_DEVOPS_ORGANIZATION,
            project = self._TEST_DEVOPS_PROJECT,
            scope = self._CONST_PROJECT_SCOPE)

        # assert
        self.mock_run_artifacttool.assert_called_with(self._TEST_DEVOPS_ORGANIZATION,
            [
                'universal', 'download',
                '--service', self._TEST_DEVOPS_ORGANIZATION,
                    '--patvar', ARTIFACTTOOL_PAT_ENVKEY,
                    '--feed', self._TEST_FEED_NAME,
                    '--package-name', self._TEST_PACKAGE_NAME,
                    '--package-version', self._TEST_PACKAGE_VERSION,
                    '--path', self._TEST_PATH,
                    '--project', self._TEST_DEVOPS_PROJECT,
                    '--filter', self._TEST_FILTER,
            ],
            'Downloading')

    def test_download_package_when_project_without_scope_then_exception(self):
        with self.assertRaises(CLIError) as exc:
            response = download_package(feed = self._TEST_FEED_NAME,
                name = self._TEST_PACKAGE_NAME,
                version = self._TEST_PACKAGE_VERSION,
                path = self._TEST_PATH,
                file_filter= self._TEST_FILTER,
                organization = self._TEST_DEVOPS_ORGANIZATION,
                project = self._TEST_DEVOPS_PROJECT)
        self.assertIn('--scope \'project\' is required when specifying a value in --project', str(exc.exception))
                


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.core.commands import CliCommandType
from azext_devops.dev.common.exception_handler import azure_devops_exception_handler
from ._format import (transform_project_table_output,
                      transform_projects_table_output,
                      transform_service_endpoints_table_output,
                      transform_authorized_service_endpoint_table_output,
                      transform_groups_table_output,
                      transform_group_table_output,
                      transform_memberships_table_output,
                      transform_membership_table_output,
                      transform_namespaces_table_output,
                      transform_namespace_table_output,
                      transform_acl_output,
                      transform_resolve_permission_bits,
                      transform_team_table_output,
                      transform_teams_table_output,
                      transform_team_members_table_output,
                      transform_users_table_output,
                      transform_user_table_output,
                      transform_extension_table_output,
                      transform_extensions_table_output,
                      transform_extension_search_results_table_output,
                      transform_wiki_table_output,
                      transform_wikis_table_output,
                      transform_wiki_page_table_output)


projectOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.project#{}',
    exception_handler=azure_devops_exception_handler
)

configureOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.configure#{}',
    exception_handler=azure_devops_exception_handler
)

feedbackOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.feedback#{}',
    exception_handler=azure_devops_exception_handler
)

credentialsOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.credentials#{}',
    exception_handler=azure_devops_exception_handler
)

service_endpointOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.service_endpoint#{}',
    exception_handler=azure_devops_exception_handler
)

teamOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.team#{}',
    exception_handler=azure_devops_exception_handler
)

invokeOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.invoke#{}',
    exception_handler=azure_devops_exception_handler
)

userOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.user#{}',
    exception_handler=azure_devops_exception_handler
)

extensionOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.extension#{}',
    exception_handler=azure_devops_exception_handler
)

security_groupOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.security_group#{}',
    exception_handler=azure_devops_exception_handler
)

security_permissionOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.security_permission#{}',
    exception_handler=azure_devops_exception_handler
)

wikiOps = CliCommandType(
    operations_tmpl='azext_devops.dev.team.wiki#{}',
    exception_handler=azure_devops_exception_handler
)


# pylint: disable=too-many-statements
def load_team_commands(self, _):
    with self.command_group('devops', command_type=credentialsOps) as g:
        g.command('login', 'credential_set')
        g.command('logout', 'credential_clear')

    with self.command_group('devops', command_type=feedbackOps) as g:
        g.command('feedback', 'feedback', deprecate_info=g.deprecate(redirect='az feedback', hide=True))

    with self.command_group('devops', command_type=configureOps) as g:
        g.command('configure', 'configure')

    with self.command_group('devops project', command_type=projectOps) as g:
        g.command('create', 'create_project', table_transformer=transform_project_table_output)
        g.command('delete', 'delete_project', confirmation='Are you sure you want to delete this project?')
        g.show_command('show', 'show_project', table_transformer=transform_project_table_output)
        g.command('list', 'list_projects', table_transformer=transform_projects_table_output)

    with self.command_group('devops service-endpoint', command_type=service_endpointOps) as g:
        g.command('list', 'list_service_endpoints', table_transformer=transform_service_endpoints_table_output)
        g.show_command('show', 'show_service_endpoint')  # no table transform because type is not well defined
        g.command('create', 'create_service_endpoint')
        g.command('update', 'update_service_endpoint',
                  table_transformer=transform_authorized_service_endpoint_table_output)
        g.command('azurerm create', 'create_azurerm_service_endpoint')
        g.command('github create', 'create_github_service_endpoint')
        g.command('delete', 'delete_service_endpoint',
                  confirmation='Are you sure you want to delete this service-endpoint?')

    with self.command_group('devops team', command_type=teamOps) as g:
        g.command('create', 'create_team', table_transformer=transform_team_table_output)
        g.command('delete', 'delete_team', confirmation='Are you sure you want to delete this team?')
        g.show_command('show', 'get_team', table_transformer=transform_team_table_output)
        g.command('list', 'get_teams', table_transformer=transform_teams_table_output)
        g.command('list-member', 'get_team_members', table_transformer=transform_team_members_table_output)
        g.command('update', 'update_team', table_transformer=transform_team_table_output)

    with self.command_group('devops', command_type=invokeOps) as g:
        g.command('invoke', 'invoke')

    with self.command_group('devops user', command_type=userOps) as g:
        g.command('list', 'get_user_entitlements', table_transformer=transform_users_table_output)
        g.show_command('show', 'get_user_entitlement', table_transformer=transform_user_table_output)
        g.command('remove', 'delete_user_entitlement', confirmation='Are you sure you want to remove this user?')
        g.command('update', 'update_user_entitlement', table_transformer=transform_user_table_output)
        g.command('add', 'add_user_entitlement', table_transformer=transform_user_table_output)

    with self.command_group('devops extension', command_type=extensionOps) as g:
        g.command('list', 'list_extensions', table_transformer=transform_extensions_table_output)
        g.command('uninstall', 'uninstall_extension', confirmation='Are you sure you want to uninstall this extension?')
        g.command('install', 'install_extension', table_transformer=transform_extension_table_output)
        g.show_command('show', 'get_extension', table_transformer=transform_extension_table_output)
        g.command('enable', 'enable_extension', table_transformer=transform_extension_table_output)
        g.command('disable', 'disable_extension', table_transformer=transform_extension_table_output)
        g.command('search', 'search_extensions', table_transformer=transform_extension_search_results_table_output)

    with self.command_group('devops security group', command_type=security_groupOps) as g:
        g.command('list', 'list_groups', table_transformer=transform_groups_table_output)
        g.show_command('show', 'get_group', table_transformer=transform_group_table_output)
        g.command('update', 'update_group', table_transformer=transform_group_table_output)
        g.command('create', 'create_group', table_transformer=transform_group_table_output)
        g.command('delete', 'delete_group', confirmation='Are you sure you want to delete this group?')

    with self.command_group('devops security group membership', command_type=security_groupOps) as g:
        g.command('list', 'list_memberships', table_transformer=transform_memberships_table_output)
        g.command('add', 'add_membership', table_transformer=transform_membership_table_output)
        g.command('remove', 'remove_membership', confirmation='Are you sure you want to delete this relationship?')

    with self.command_group('devops security permission', command_type=security_permissionOps) as g:
        g.command('list', 'list_tokens', table_transformer=transform_acl_output)
        g.command('update', 'update_permissions', table_transformer=transform_resolve_permission_bits)
        g.command('reset-all', 'reset_all_permissions',
                  confirmation='Are you sure you want to reset all explicit permissions for this user/group and token?')
        g.command('reset', 'reset_permissions', table_transformer=transform_resolve_permission_bits)
        g.show_command('show', 'show_permissions', table_transformer=transform_resolve_permission_bits)

    with self.command_group('devops security permission namespace', command_type=security_permissionOps) as g:
        g.command('list', 'list_namespaces', table_transformer=transform_namespaces_table_output)
        g.show_command('show', 'show_namespace', table_transformer=transform_namespace_table_output)

    with self.command_group('devops wiki', command_type=wikiOps) as g:
        g.command('create', 'create_wiki', table_transformer=transform_wiki_table_output)
        g.command('list', 'list_wiki', table_transformer=transform_wikis_table_output)
        g.show_command('show', 'show_wiki', table_transformer=transform_wiki_table_output)
        g.command('delete', 'delete_wiki', table_transformer=transform_wiki_table_output,
                  confirmation='Are you sure you want to delete this wiki?')

    with self.command_group('devops wiki page', command_type=wikiOps) as g:
        g.command('create', 'add_page', table_transformer=transform_wiki_page_table_output)
        g.command('update', 'update_page', table_transformer=transform_wiki_page_table_output)
        g.show_command('show', 'get_page', table_transformer=transform_wiki_page_table_output)
        g.command('delete', 'delete_page',
                  confirmation='Are you sure you want to delete this wiki page?')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.cli.core._environment import get_config_dir

# Config directory
AZ_CLI_GLOBAL_CONFIG_DIR = get_config_dir()
_AZ_DEVOPS_CONFIG_DIR_NAME = 'azuredevops'
AZ_DEVOPS_DEFAULT_CONFIG_DIR = os.path.join(AZ_CLI_GLOBAL_CONFIG_DIR, _AZ_DEVOPS_CONFIG_DIR_NAME)


# Environment Variables
CLI_ENV_VARIABLE_PREFIX = 'AZURE_DEVOPS_EXT_'
AZ_DEVOPS_CONFIG_DIR_ENVKEY = CLI_ENV_VARIABLE_PREFIX + 'CONFIG_DIR'
AZ_DEVOPS_GITHUB_PAT_ENVKEY = CLI_ENV_VARIABLE_PREFIX + 'GITHUB_PAT'
AZ_DEVOPS_PIPELINES_VARIABLES_KEY_PREFIX = CLI_ENV_VARIABLE_PREFIX + 'PIPELINE_VAR_'

# Import request Environment Variable
GIT_SOURCE_PASSWORD_OR_PAT = CLI_ENV_VARIABLE_PREFIX + 'GIT_SOURCE_PASSWORD_OR_PAT'

# Config file constants
CONFIG_FILE_NAME = 'config'
DEFAULTS_SECTION = 'defaults'
DEVOPS_ORGANIZATION_DEFAULT = 'organization'
DEVOPS_TEAM_PROJECT_DEFAULT = 'project'
PAT_ENV_VARIABLE_NAME = CLI_ENV_VARIABLE_PREFIX + 'PAT'
AUTH_TOKEN_ENV_VARIABLE_NAME = CLI_ENV_VARIABLE_PREFIX + 'AUTH_TOKEN'

# Artifact tool variables
ARTIFACTTOOL_OVERRIDE_PATH_ENVKEY = CLI_ENV_VARIABLE_PREFIX + "ARTIFACTTOOL_OVERRIDE_PATH"
ARTIFACTTOOL_OVERRIDE_URL_ENVKEY = CLI_ENV_VARIABLE_PREFIX + "ARTIFACTTOOL_OVERRIDE_URL"
ARTIFACTTOOL_OVERRIDE_VERSION_ENVKEY = CLI_ENV_VARIABLE_PREFIX + "ARTIFACTTOOL_OVERRIDE_VERSION"
ARTIFACTTOOL_PAT_ENVKEY = CLI_ENV_VARIABLE_PREFIX + "ARTIFACTTOOL_PATVAR"

# Telemetry Constants
ORG_PRESENT_IN_COMMAND = 'OrgPresentInCommand'
PROJECT_PRESENT_IN_COMMAND = 'ProjectPresentInCommand'
REPO_PRESENT_IN_COMMAND = 'RepoPresentInCommand'
ORG_PICKED_FROM_GIT = 'OrgPickedFromGit'
PROJECT_PICKED_FROM_GIT = 'ProjectPickedFromGit'
REPO_PICKED_FROM_GIT = 'RepoPickedFromGit'
ORG_PICKED_FROM_CONFIG = 'OrgPickedFromConfig'
ORG_IGNORED_FROM_CONFIG = 'OrgIgnoredFromConfig'
PROJECT_PICKED_FROM_CONFIG = 'ProjectPickedFromConfig'
PROJECT_IGNORED_FROM_CONFIG = 'ProjectIgnoredFromConfig'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger
from azext_devops.dev.common.services import resolve_instance_and_project, get_build_client
from azext_devops.dev.common.git import resolve_git_ref_heads
from azext_devops.dev.common.identities import resolve_identity_as_id

logger = get_logger(__name__)


def pipeline_run_show(id, open=False, organization=None, project=None, detect=None):  # pylint: disable=redefined-builtin
    """ Show details of a pipeline run.
    :param id: ID of the pipeline run.
    :type id: int
    :param open: Open the build results page in your web browser.
    :type open: bool
    :param project: Name or ID of the team project.
    :type project: str
    :param detect: Automatically detect organization and project. Default is "on".
    :type detect: str
    :rtype: :class:`<Build> <build.v5_1.models.Build>`
    """
    organization, project = resolve_instance_and_project(
        detect=detect, organization=organization, project=project)
    client = get_build_client(organization)
    build = client.get_build(build_id=id, project=project)
    if open:
        _open_pipeline_run(build, organization)
    return build


def pipeline_run_list(pipeline_ids=None, branch=None, organization=None, project=None, detect=None, top=None,
                      query_order=None, result=None, status=None, reason=None, tags=None, requested_for=None):
    """ List the pipeline runs in a project.
    :param pipeline_ids: IDs (space separated) of definitions to list builds for.
    For multiple pipeline ids:  --pipeline-ids 1 2
    :type pipeline_ids: list of int
    :param branch: Filter by builds for this branch.
    :type branch: str
    :param top: Maximum number of builds to list.
    :type top: int
    :param query_order: Order of pipeline runs.
    :type query_order: str
    :param result: Limit to builds with this result.
    :type result: str
    :param status: Limit to builds with this status.
    :type status: str
    :param reason: Limit to builds with this reason.
    :type reason: str
    :param tags: Limit to builds with each of the specified tags. Space separated.
    :type tags: list of str
    :param requested_for: Limit to builds requested for this user or group.
    :type requested_for: str
    """
    organization, project = resolve_instance_and_project(
        detect=detect, organization=organization, project=project)
    client = get_build_client(organization)
    if pipeline_ids is not None and pipeline_ids:
        pipeline_ids = list(set(pipeline_ids))  # make distinct
    if tags is not None and tags:
        tags = list(set(tags))  # make distinct
    query_order = _resolve_runs_query_order(query_order)
    builds = client.get_builds(definitions=pipeline_ids,
                               project=project,
                               branch_name=resolve_git_ref_heads(branch),
                               top=top,
                               result_filter=result,
                               status_filter=status,
                               reason_filter=reason,
                               tag_filters=tags,
                               query_order=query_order,
                               requested_for=resolve_identity_as_id(requested_for, organization))
    return builds


def _resolve_runs_query_order(query_order):
    if query_order:
        query_order_vals = ['finishTimeAscending', 'finishTimeDescending', 'queueTimeAscending',
                            'queueTimeDescending', 'startTimeAscending', 'startTimeDescending']
        for val in query_order_vals:
            if query_order.lower() in val.lower():
                return val
        logger.warning("Cannot resolve --query-order, continuing with None")
    return None


def pipeline_run_add_tag(run_id, tags, organization=None, project=None, detect=None):
    """ Add tag(s) for a pipeline run.
    :param run_id: ID of the pipeline run.
    :type run_id: int
    :param tags: Tag(s) to be added to the pipeline run. [Comma separated values]
    :type tags: str
    :rtype: list of str
    """
    organization, project = resolve_instance_and_project(detect=detect,
                                                         organization=organization,
                                                         project=project)
    client = get_build_client(organization)
    tags = list(map(str, tags.split(',')))
    if len(tags) == 1:
        tags = client.add_build_tag(
            project=project, build_id=run_id, tag=tags[0])
    else:
        tags = client.add_build_tags(
            tags=tags, project=project, build_id=run_id)
    return tags


def pipeline_run_delete_tag(run_id, tag, organization=None, project=None, detect=None):
    """ Delete a pipeline run tag.
    :param run_id: ID of the pipeline run.
    :type run_id: int
    :param tag: Tag to be deleted from the pipeline run.
    :type tag: str
    :rtype: list of str
    """
    organization, project = resolve_instance_and_project(detect=detect,
                                                         organization=organization,
                                                         project=project)
    client = get_build_client(organization)
    tags = client.delete_build_tag(project=project, build_id=run_id, tag=tag)
    return tags


def pipeline_run_get_tags(run_id, organization=None, project=None, detect=None):
    """ Get tags for a pipeline run.
    :param run_id: ID of the  pipeline run.
    :type run_id: int
    :rtype: list of str
    """
    organization, project = resolve_instance_and_project(detect=detect,
                                                         organization=organization,
                                                         project=project)
    client = get_build_client(organization)
    tags = client.get_build_tags(build_id=run_id, project=project)
    return tags


def _open_pipeline_run(run, organization):
    """Open the build results page in your web browser.
    :param :class:`<Build> <build.v5_1.models.Build>` build:
    :param str organization:
    """
    from webbrowser import open_new
    from azext_devops.dev.common.uri import uri_quote
    # https://dev.azure.com/OrgName/ProjectName/_build/results?buildId=1234
    project = run.project.name
    url = organization.rstrip('/') + '/' + uri_quote(project) + '/_build/results?buildid='\
        + uri_quote(str(run.id))
    logger.debug('Opening web page: %s', url)
    open_new(url=url)


def _open_pipeline_run6_0(run, project, organization):
    """Open the build results page in your web browser.
    :param :class:`<Run> <azure.devops.v6_0.pipelines.models.Run>`
    :param str project:
    :param str organization:
    """
    from webbrowser import open_new
    from azext_devops.dev.common.uri import uri_quote
    url = f"{organization.rstrip('/')}/{uri_quote(project)}/_build/results?buildid={uri_quote(str(run.id))}"
    logger.debug('Opening web page: %s', url)
    open_new(url=url)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

try:
    # Attempt to load mock (works on Python 3.3 and above)
    from unittest.mock import patch
except ImportError:
    # Attempt to load mock (works on Python version below 3.3)
    from mock import patch

from azext_devops.dev.common.services import clear_connection_cache
from azext_devops.test.utils.authentication import AuthenticatedTests

from azext_devops.devops_sdk.v5_0.build.build_client import BuildClient
from azext_devops.devops_sdk.v6_0.pipelines.pipelines_client import PipelinesClient
from azext_devops.dev.pipelines.pipeline import pipeline_run
from azext_devops.test.utils.helper import get_client_mock_helper, TEST_DEVOPS_ORG_URL

class TestPipelinesRun(AuthenticatedTests):

    _TEST_DEVOPS_ORGANIZATION = 'https://someorganization.visualstudio.com'
    _TEST_DEVOPS_PROJECT = 'MyProject'
    _PARAMETERS = ['yesNo=FromtheCode']

    def setUp(self):
        self.authentication_setup()
        self.authenticate()

        clear_connection_cache()

    def tearDown(self):
        patch.stopall()

    def test_pr_run_without_parameters(self):
        with patch('azext_devops.devops_sdk.connection.Connection.get_client', new=get_client_mock_helper) as get_build_client:
            with patch('azext_devops.devops_sdk.v5_0.build.build_client.BuildClient.queue_build') as mock_queue_build:
                with patch('azext_devops.devops_sdk.v6_0.pipelines.pipelines_client.PipelinesClient.run_pipeline') as mock_run_pipeline:
                    get_build_client.return_value = BuildClient(base_url=self._TEST_DEVOPS_ORGANIZATION)
                    pipeline_run(id=1, project=self._TEST_DEVOPS_PROJECT,organization=self._TEST_DEVOPS_ORGANIZATION)

                    mock_queue_build.assert_called_once()
                    mock_run_pipeline.assert_not_called()

    def test_pr_run_with_parameters(self):
        with patch('azext_devops.devops_sdk.connection.Connection.get_client', new=get_client_mock_helper) as get_build_client:
            with patch('azext_devops.devops_sdk.connection.Connection.get_client') as get_new_pipeline_client_v60:
                with patch('azext_devops.devops_sdk.v6_0.pipelines.pipelines_client.PipelinesClient.run_pipeline') as mock_run_pipeline:
                    with patch('azext_devops.devops_sdk.v5_0.build.build_client.BuildClient.queue_build') as mock_queue_build:
                        get_build_client.return_value = BuildClient(base_url=self._TEST_DEVOPS_ORGANIZATION)
                        get_new_pipeline_client_v60.return_value = PipelinesClient(base_url=self._TEST_DEVOPS_ORGANIZATION)

                        pipeline_run(id=1, project=self._TEST_DEVOPS_PROJECT,
                                    organization=self._TEST_DEVOPS_ORGANIZATION,parameters=self._PARAMETERS)

                        mock_queue_build.assert_not_called()
                        mock_run_pipeline.assert_called_once()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class BoardColumnBase(Model):
    """BoardColumnBase.

    :param description: Board column description.
    :type description: str
    :param name: Name of the column.
    :type name: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'}
    }

    def __init__(self, description=None, name=None):
        super(BoardColumnBase, self).__init__()
        self.description = description
        self.name = name


class BoardColumnCollectionResponse(Model):
    """BoardColumnCollectionResponse.

    :param _links: Links to other related objects.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.boards.models.ReferenceLinks>`
    :param board_columns: The resulting collection of BoardColumn.
    :type board_columns: list of :class:`BoardColumn <azure.devops.v5_0.boards.models.BoardColumn>`
    :param eTag: The last change date and time for all the columns in the collection.
    :type eTag: list of str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'board_columns': {'key': 'boardColumns', 'type': '[BoardColumn]'},
        'eTag': {'key': 'eTag', 'type': '[str]'}
    }

    def __init__(self, _links=None, board_columns=None, eTag=None):
        super(BoardColumnCollectionResponse, self).__init__()
        self._links = _links
        self.board_columns = board_columns
        self.eTag = eTag


class BoardColumnCreate(BoardColumnBase):
    """BoardColumnCreate.

    :param description: Board column description.
    :type description: str
    :param name: Name of the column.
    :type name: str
    :param next_column_id: Next column identifier or supported directive: $first or $last.
    :type next_column_id: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'next_column_id': {'key': 'nextColumnId', 'type': 'str'}
    }

    def __init__(self, description=None, name=None, next_column_id=None):
        super(BoardColumnCreate, self).__init__(description=description, name=name)
        self.next_column_id = next_column_id


class BoardColumnResponse(Model):
    """BoardColumnResponse.

    :param board_column: The resulting BoardColumn.
    :type board_column: :class:`BoardColumn <azure.devops.v5_0.boards.models.BoardColumn>`
    :param eTag: The last change date and time for all the columns in the collection.
    :type eTag: list of str
    """

    _attribute_map = {
        'board_column': {'key': 'boardColumn', 'type': 'BoardColumn'},
        'eTag': {'key': 'eTag', 'type': '[str]'}
    }

    def __init__(self, board_column=None, eTag=None):
        super(BoardColumnResponse, self).__init__()
        self.board_column = board_column
        self.eTag = eTag


class BoardColumnUpdate(BoardColumnCreate):
    """BoardColumnUpdate.

    :param description: Board column description.
    :type description: str
    :param next_column_id: Next column identifier or supported directive: $first or $last.
    :type next_column_id: str
    :param name: Name of the column.
    :type name: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'next_column_id': {'key': 'nextColumnId', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'}
    }

    def __init__(self, description=None, next_column_id=None, name=None):
        super(BoardColumnUpdate, self).__init__(description=description, next_column_id=next_column_id)
        self.name = name


class BoardItemCollectionResponse(Model):
    """BoardItemCollectionResponse.

    :param _links: Links to other related objects.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.boards.models.ReferenceLinks>`
    :param board_items: The resulting collection of BoardItem.
    :type board_items: list of :class:`BoardItem <azure.devops.v5_0.boards.models.BoardItem>`
    :param eTag: The last change date and time for all items in the collection.
    :type eTag: list of str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'board_items': {'key': 'boardItems', 'type': '[BoardItem]'},
        'eTag': {'key': 'eTag', 'type': '[str]'}
    }

    def __init__(self, _links=None, board_items=None, eTag=None):
        super(BoardItemCollectionResponse, self).__init__()
        self._links = _links
        self.board_items = board_items
        self.eTag = eTag


class BoardItemIdAndType(Model):
    """BoardItemIdAndType.

    :param item_id: Item id.
    :type item_id: str
    :param item_type: Item type.
    :type item_type: str
    """

    _attribute_map = {
        'item_id': {'key': 'itemId', 'type': 'str'},
        'item_type': {'key': 'itemType', 'type': 'str'}
    }

    def __init__(self, item_id=None, item_type=None):
        super(BoardItemIdAndType, self).__init__()
        self.item_id = item_id
        self.item_type = item_type


class BoardItemReference(BoardItemIdAndType):
    """BoardItemReference.

    :param item_id: Item id.
    :type item_id: str
    :param item_type: Item type.
    :type item_type: str
    :param unique_id: Board's unique identifier. Compound identifier generated using the item identifier and item type.
    :type unique_id: str
    :param url: Full http link to the resource.
    :type url: str
    """

    _attribute_map = {
        'item_id': {'key': 'itemId', 'type': 'str'},
        'item_type': {'key': 'itemType', 'type': 'str'},
        'unique_id': {'key': 'uniqueId', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, item_id=None, item_type=None, unique_id=None, url=None):
        super(BoardItemReference, self).__init__(item_id=item_id, item_type=item_type)
        self.unique_id = unique_id
        self.url = url


class BoardItemResponse(Model):
    """BoardItemResponse.

    :param eTag: The last changed date for the board item.
    :type eTag: list of str
    :param item: The resulting BoardItem.
    :type item: :class:`BoardItem <azure.devops.v5_0.boards.models.BoardItem>`
    """

    _attribute_map = {
        'eTag': {'key': 'eTag', 'type': '[str]'},
        'item': {'key': 'item', 'type': 'BoardItem'}
    }

    def __init__(self, eTag=None, item=None):
        super(BoardItemResponse, self).__init__()
        self.eTag = eTag
        self.item = item


class BoardResponse(Model):
    """BoardResponse.

    :param board: The resulting Board.
    :type board: :class:`Board <azure.devops.v5_0.boards.models.Board>`
    :param eTag: The last date and time the board was changed.
    :type eTag: list of str
    """

    _attribute_map = {
        'board': {'key': 'board', 'type': 'Board'},
        'eTag': {'key': 'eTag', 'type': '[str]'}
    }

    def __init__(self, board=None, eTag=None):
        super(BoardResponse, self).__init__()
        self.board = board
        self.eTag = eTag


class BoardRowBase(Model):
    """BoardRowBase.

    :param name: Row name.
    :type name: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'}
    }

    def __init__(self, name=None):
        super(BoardRowBase, self).__init__()
        self.name = name


class BoardRowCollectionResponse(Model):
    """BoardRowCollectionResponse.

    :param _links: Links to other related objects.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.boards.models.ReferenceLinks>`
    :param board_rows: The resulting collection of BoardRow.
    :type board_rows: list of :class:`BoardRow <azure.devops.v5_0.boards.models.BoardRow>`
    :param eTag: The last change date and time for all the rows in the collection.
    :type eTag: list of str
    """

    _attribute_map = {
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'board_rows': {'key': 'boardRows', 'type': '[BoardRow]'},
        'eTag': {'key': 'eTag', 'type': '[str]'}
    }

    def __init__(self, _links=None, board_rows=None, eTag=None):
        super(BoardRowCollectionResponse, self).__init__()
        self._links = _links
        self.board_rows = board_rows
        self.eTag = eTag


class BoardRowCreate(BoardRowBase):
    """BoardRowCreate.

    :param name: Row name.
    :type name: str
    :param next_row_id: Next row identifier or supported directive: $first or $last.
    :type next_row_id: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'next_row_id': {'key': 'nextRowId', 'type': 'str'}
    }

    def __init__(self, name=None, next_row_id=None):
        super(BoardRowCreate, self).__init__(name=name)
        self.next_row_id = next_row_id


class BoardRowResponse(Model):
    """BoardRowResponse.

    :param board_row: The resulting collection of BoardRow.
    :type board_row: :class:`BoardRow <azure.devops.v5_0.boards.models.BoardRow>`
    :param eTag: The last change date and time for all the rows in the collection.
    :type eTag: list of str
    """

    _attribute_map = {
        'board_row': {'key': 'boardRow', 'type': 'BoardRow'},
        'eTag': {'key': 'eTag', 'type': '[str]'}
    }

    def __init__(self, board_row=None, eTag=None):
        super(BoardRowResponse, self).__init__()
        self.board_row = board_row
        self.eTag = eTag


class BoardRowUpdate(BoardRowCreate):
    """BoardRowUpdate.

    :param name: Row name.
    :type name: str
    :param next_row_id: Next row identifier or supported directive: $first or $last.
    :type next_row_id: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'next_row_id': {'key': 'nextRowId', 'type': 'str'},
    }

    def __init__(self, name=None, next_row_id=None):
        super(BoardRowUpdate, self).__init__(name=name, next_row_id=next_row_id)


class CreateBoard(Model):
    """CreateBoard.

    :param description: Description of the board.
    :type description: str
    :param name: Name of the board to create.
    :type name: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'}
    }

    def __init__(self, description=None, name=None):
        super(CreateBoard, self).__init__()
        self.description = description
        self.name = name


class EntityReference(Model):
    """EntityReference.

    :param name: Name of the resource.
    :type name: str
    :param url: Full http link to the resource.
    :type url: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, name=None, url=None):
        super(EntityReference, self).__init__()
        self.name = name
        self.url = url


class NewBoardItem(BoardItemIdAndType):
    """NewBoardItem.

    :param item_id: Item id.
    :type item_id: str
    :param item_type: Item type.
    :type item_type: str
    :param column_id: Board column identifier.
    :type column_id: str
    :param next_item_unique_id: Next item unique identifier or supported directive: $first or $last.
    :type next_item_unique_id: str
    :param row_id: Board row identifier.
    :type row_id: str
    """

    _attribute_map = {
        'item_id': {'key': 'itemId', 'type': 'str'},
        'item_type': {'key': 'itemType', 'type': 'str'},
        'column_id': {'key': 'columnId', 'type': 'str'},
        'next_item_unique_id': {'key': 'nextItemUniqueId', 'type': 'str'},
        'row_id': {'key': 'rowId', 'type': 'str'}
    }

    def __init__(self, item_id=None, item_type=None, column_id=None, next_item_unique_id=None, row_id=None):
        super(NewBoardItem, self).__init__(item_id=item_id, item_type=item_type)
        self.column_id = column_id
        self.next_item_unique_id = next_item_unique_id
        self.row_id = row_id


class ReferenceLinks(Model):
    """ReferenceLinks.

    :param links: The readonly view of the links.  Because Reference links are readonly, we only want to expose them as read only.
    :type links: dict
    """

    _attribute_map = {
        'links': {'key': 'links', 'type': '{object}'}
    }

    def __init__(self, links=None):
        super(ReferenceLinks, self).__init__()
        self.links = links


class UpdateBoard(Model):
    """UpdateBoard.

    :param description: New description of the board.
    :type description: str
    :param name: New name of the board.
    :type name: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'}
    }

    def __init__(self, description=None, name=None):
        super(UpdateBoard, self).__init__()
        self.description = description
        self.name = name


class UpdateBoardItem(Model):
    """UpdateBoardItem.

    :param column_id: Board column identifier.
    :type column_id: str
    :param next_item_unique_id: Next unique item identifier or supported directive: $first or $last.
    :type next_item_unique_id: str
    :param row_id: Board row identifier.
    :type row_id: str
    """

    _attribute_map = {
        'column_id': {'key': 'columnId', 'type': 'str'},
        'next_item_unique_id': {'key': 'nextItemUniqueId', 'type': 'str'},
        'row_id': {'key': 'rowId', 'type': 'str'}
    }

    def __init__(self, column_id=None, next_item_unique_id=None, row_id=None):
        super(UpdateBoardItem, self).__init__()
        self.column_id = column_id
        self.next_item_unique_id = next_item_unique_id
        self.row_id = row_id


class BoardColumn(BoardColumnBase):
    """BoardColumn.

    :param description: Board column description.
    :type description: str
    :param name: Name of the column.
    :type name: str
    :param _links: Links to other related objects.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.boards.models.ReferenceLinks>`
    :param id: Id of the resource.
    :type id: str
    :param next_column_id: Next column identifier.
    :type next_column_id: str
    :param url: Full http link to the resource.
    :type url: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'id': {'key': 'id', 'type': 'str'},
        'next_column_id': {'key': 'nextColumnId', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, description=None, name=None, _links=None, id=None, next_column_id=None, url=None):
        super(BoardColumn, self).__init__(description=description, name=name)
        self._links = _links
        self.id = id
        self.next_column_id = next_column_id
        self.url = url


class BoardItem(BoardItemReference):
    """BoardItem.

    :param item_id: Item id.
    :type item_id: str
    :param item_type: Item type.
    :type item_type: str
    :param unique_id: Board's unique identifier. Compound identifier generated using the item identifier and item type.
    :type unique_id: str
    :param url: Full http link to the resource.
    :type url: str
    :param _links: Links to other related objects.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.boards.models.ReferenceLinks>`
    :param board_id: Board id for this item.
    :type board_id: int
    :param column: Board column id for this item.
    :type column: str
    :param next_item_unique_id: Next item unique identifier.
    :type next_item_unique_id: str
    :param row: Board row id for this item.
    :type row: str
    """

    _attribute_map = {
        'item_id': {'key': 'itemId', 'type': 'str'},
        'item_type': {'key': 'itemType', 'type': 'str'},
        'unique_id': {'key': 'uniqueId', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'board_id': {'key': 'boardId', 'type': 'int'},
        'column': {'key': 'column', 'type': 'str'},
        'next_item_unique_id': {'key': 'nextItemUniqueId', 'type': 'str'},
        'row': {'key': 'row', 'type': 'str'}
    }

    def __init__(self, item_id=None, item_type=None, unique_id=None, url=None, _links=None, board_id=None, column=None, next_item_unique_id=None, row=None):
        super(BoardItem, self).__init__(item_id=item_id, item_type=item_type, unique_id=unique_id, url=url)
        self._links = _links
        self.board_id = board_id
        self.column = column
        self.next_item_unique_id = next_item_unique_id
        self.row = row


class BoardReference(EntityReference):
    """BoardReference.

    :param name: Name of the resource.
    :type name: str
    :param url: Full http link to the resource.
    :type url: str
    :param id: Id of the resource.
    :type id: int
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        'id': {'key': 'id', 'type': 'int'}
    }

    def __init__(self, name=None, url=None, id=None):
        super(BoardReference, self).__init__(name=name, url=url)
        self.id = id


class BoardRow(BoardRowBase):
    """BoardRow.

    :param name: Row name.
    :type name: str
    :param _links: Links to other related objects.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.boards.models.ReferenceLinks>`
    :param id: Id of the resource.
    :type id: str
    :param next_row_id: Next row identifier.
    :type next_row_id: str
    :param url: Full http link to the resource.
    :type url: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'id': {'key': 'id', 'type': 'str'},
        'next_row_id': {'key': 'nextRowId', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'}
    }

    def __init__(self, name=None, _links=None, id=None, next_row_id=None, url=None):
        super(BoardRow, self).__init__(name=name)
        self._links = _links
        self.id = id
        self.next_row_id = next_row_id
        self.url = url


class Board(BoardReference):
    """Board.

    :param name: Name of the resource.
    :type name: str
    :param url: Full http link to the resource.
    :type url: str
    :param id: Id of the resource.
    :type id: int
    :param _links: Links to other related objects.
    :type _links: :class:`ReferenceLinks <azure.devops.v5_0.boards.models.ReferenceLinks>`
    :param description: Description of the board.
    :type description: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        'id': {'key': 'id', 'type': 'int'},
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'description': {'key': 'description', 'type': 'str'}
    }

    def __init__(self, name=None, url=None, id=None, _links=None, description=None):
        super(Board, self).__init__(name=name, url=url, id=id)
        self._links = _links
        self.description = description


__all__ = [
    'BoardColumnBase',
    'BoardColumnCollectionResponse',
    'BoardColumnCreate',
    'BoardColumnResponse',
    'BoardColumnUpdate',
    'BoardItemCollectionResponse',
    'BoardItemIdAndType',
    'BoardItemReference',
    'BoardItemResponse',
    'BoardResponse',
    'BoardRowBase',
    'BoardRowCollectionResponse',
    'BoardRowCreate',
    'BoardRowResponse',
    'BoardRowUpdate',
    'CreateBoard',
    'EntityReference',
    'NewBoardItem',
    'ReferenceLinks',
    'UpdateBoard',
    'UpdateBoardItem',
    'BoardColumn',
    'BoardItem',
    'BoardReference',
    'BoardRow',
    'Board',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest

from azure_devtools.scenario_tests import AllowLargeResponse
from .utilities.helper import DevopsScenarioTest, disable_telemetry, set_authentication, get_test_org_from_env_variable

DEVOPS_CLI_TEST_ORGANIZATION = get_test_org_from_env_variable() or 'Https://dev.azure.com/v-anvashist0376'


class BoardsQueryTests(DevopsScenarioTest): 
    @AllowLargeResponse(size_kb=3072)
    @disable_telemetry
    @set_authentication
    def test_queryShow(self):
        wi_test_project_name = 'WorkItemCreateShowUpdateDeleteTests'
        self.cmd('az devops configure --defaults organization=' + DEVOPS_CLI_TEST_ORGANIZATION)

        show_query_command_with_id = 'az boards query --org '+ DEVOPS_CLI_TEST_ORGANIZATION +' --detect false -p ' + wi_test_project_name + ' --id 253d8eb4-0bd1-4c92-adc9-d305c8b99b92 --output json'
        query_result = self.cmd(show_query_command_with_id).get_output_in_json()
        assert len(query_result) > 0

        wiql_string = "select [System.Id], [System.WorkItemType], [System.Title], [System.AssignedTo], [System.State], [System.Tags] from WorkItems where [System.TeamProject] = 'WorkItemCreateShowUpdateDeleteTests' and [System.WorkItemType] = 'Task'"
        show_query_command_with_wiql = 'az boards query --org ' + DEVOPS_CLI_TEST_ORGANIZATION +' -p WorkItemCreateShowUpdateDeleteTests --wiql "' + wiql_string +'" --output json'
        query_result = self.cmd(show_query_command_with_wiql).get_output_in_json()
        assert len(query_result) > 0


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest

try:
    # Attempt to load mock (works on Python 3.3 and above)
    from unittest.mock import patch
except ImportError:
    # Attempt to load mock (works on Python version below 3.3)
    from mock import patch
from .helper import UNIT_TEST_PAT_TOKEN

class AuthenticatedTests(unittest.TestCase):

    def authentication_setup(self):
        self.resolve_identity_patcher = patch('azext_devops.dev.common.identities.resolve_identity_as_id') # pylint: disable=attribute-defined-outside-init
        self.get_credential_patcher = patch('azext_devops.dev.common.services.get_credential') # pylint: disable=attribute-defined-outside-init
        self.validate_token_patcher = patch('azext_devops.dev.common.services.validate_token_for_instance') # pylint: disable=attribute-defined-outside-init

        # start the patchers
        self.mock_resolve_identity = self.resolve_identity_patcher.start() # pylint: disable=attribute-defined-outside-init
        self.mock_get_credential = self.get_credential_patcher.start() # pylint: disable=attribute-defined-outside-init
        self.mock_validate_token = self.validate_token_patcher.start() # pylint: disable=attribute-defined-outside-init

    def authenticate(self):
        # set return values
        self.mock_validate_token.return_value = True
        self.mock_get_credential.return_value = UNIT_TEST_PAT_TOKEN



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys

from azure.cli import __main__ as cli_main

# If import here is failing then it means you need to install azure cli core in machine
# pip install "git+https://github.com/Azure/azure-cli@dev#egg=azure-cli-core&subdirectory=src/azure-cli-core"

sys.exit(cli_main(sys.argv))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest import Serializer, Deserializer
from ...client import Client
from . import models


class TestPlanClient(Client):
    """TestPlan
    :param str base_url: Service URL
    :param Authentication creds: Authenticated credentials.
    """

    def __init__(self, base_url=None, creds=None):
        super(TestPlanClient, self).__init__(base_url, creds)
        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

    resource_area_identifier = None

    def create_test_configuration(self, test_configuration_create_update_parameters, project):
        """CreateTestConfiguration.
        [Preview API] Create a test configuration.
        :param :class:`<TestConfigurationCreateUpdateParameters> <azure.devops.v5_1.test_plan.models.TestConfigurationCreateUpdateParameters>` test_configuration_create_update_parameters: TestConfigurationCreateUpdateParameters
        :param str project: Project ID or project name
        :rtype: :class:`<TestConfiguration> <azure.devops.v5_1.test_plan.models.TestConfiguration>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        content = self._serialize.body(test_configuration_create_update_parameters, 'TestConfigurationCreateUpdateParameters')
        response = self._send(http_method='POST',
                              location_id='8369318e-38fa-4e84-9043-4b2a75d2c256',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('TestConfiguration', response)

    def delete_test_confguration(self, project, test_configuartion_id):
        """DeleteTestConfguration.
        [Preview API] Delete a test configuration by its ID.
        :param str project: Project ID or project name
        :param int test_configuartion_id: ID of the test configuration to delete.
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        query_parameters = {}
        if test_configuartion_id is not None:
            query_parameters['testConfiguartionId'] = self._serialize.query('test_configuartion_id', test_configuartion_id, 'int')
        self._send(http_method='DELETE',
                   location_id='8369318e-38fa-4e84-9043-4b2a75d2c256',
                   version='5.1-preview.1',
                   route_values=route_values,
                   query_parameters=query_parameters)

    def get_test_configuration_by_id(self, project, test_configuration_id):
        """GetTestConfigurationById.
        [Preview API] Get a test configuration
        :param str project: Project ID or project name
        :param int test_configuration_id: ID of the test configuration to get.
        :rtype: :class:`<TestConfiguration> <azure.devops.v5_1.test_plan.models.TestConfiguration>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if test_configuration_id is not None:
            route_values['testConfigurationId'] = self._serialize.url('test_configuration_id', test_configuration_id, 'int')
        response = self._send(http_method='GET',
                              location_id='8369318e-38fa-4e84-9043-4b2a75d2c256',
                              version='5.1-preview.1',
                              route_values=route_values)
        return self._deserialize('TestConfiguration', response)

    def get_test_configurations(self, project, continuation_token=None):
        """GetTestConfigurations.
        [Preview API] Get a list of test configurations.
        :param str project: Project ID or project name
        :param str continuation_token: If the list of configurations returned is not complete, a continuation token to query next batch of configurations is included in the response header as "x-ms-continuationtoken". Omit this parameter to get the first batch of test configurations.
        :rtype: :class:`<GetTestConfigurationsResponseValue>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        query_parameters = {}
        if continuation_token is not None:
            query_parameters['continuationToken'] = self._serialize.query('continuation_token', continuation_token, 'str')
        response = self._send(http_method='GET',
                              location_id='8369318e-38fa-4e84-9043-4b2a75d2c256',
                              version='5.1-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        response_value = self._deserialize('[TestConfiguration]', self._unwrap_collection(response))
        continuation_token = self._get_continuation_token(response)
        return self.GetTestConfigurationsResponseValue(response_value, continuation_token)

    class GetTestConfigurationsResponseValue(object):
        def __init__(self, value, continuation_token):
            """
            Response for the get_test_configurations method

            :param value:
            :type value: :class:`<[TestConfiguration]> <azure.devops.v5_1.test_plan.models.[TestConfiguration]>`
            :param continuation_token: The continuation token to be used to get the next page of results.
            :type continuation_token: str
            """
            self.value = value
            self.continuation_token = continuation_token

    def update_test_configuration(self, test_configuration_create_update_parameters, project, test_configuartion_id):
        """UpdateTestConfiguration.
        [Preview API] Update a test configuration by its ID.
        :param :class:`<TestConfigurationCreateUpdateParameters> <azure.devops.v5_1.test_plan.models.TestConfigurationCreateUpdateParameters>` test_configuration_create_update_parameters: TestConfigurationCreateUpdateParameters
        :param str project: Project ID or project name
        :param int test_configuartion_id: ID of the test configuration to update.
        :rtype: :class:`<TestConfiguration> <azure.devops.v5_1.test_plan.models.TestConfiguration>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        query_parameters = {}
        if test_configuartion_id is not None:
            query_parameters['testConfiguartionId'] = self._serialize.query('test_configuartion_id', test_configuartion_id, 'int')
        content = self._serialize.body(test_configuration_create_update_parameters, 'TestConfigurationCreateUpdateParameters')
        response = self._send(http_method='PATCH',
                              location_id='8369318e-38fa-4e84-9043-4b2a75d2c256',
                              version='5.1-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters,
                              content=content)
        return self._deserialize('TestConfiguration', response)

    def create_test_plan(self, test_plan_create_params, project):
        """CreateTestPlan.
        [Preview API] Create a test plan.
        :param :class:`<TestPlanCreateParams> <azure.devops.v5_1.test_plan.models.TestPlanCreateParams>` test_plan_create_params: A testPlanCreateParams object.TestPlanCreateParams
        :param str project: Project ID or project name
        :rtype: :class:`<TestPlan> <azure.devops.v5_1.test_plan.models.TestPlan>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        content = self._serialize.body(test_plan_create_params, 'TestPlanCreateParams')
        response = self._send(http_method='POST',
                              location_id='0e292477-a0c2-47f3-a9b6-34f153d627f4',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('TestPlan', response)

    def delete_test_plan(self, project, plan_id):
        """DeleteTestPlan.
        [Preview API] Delete a test plan.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan to be deleted.
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        self._send(http_method='DELETE',
                   location_id='0e292477-a0c2-47f3-a9b6-34f153d627f4',
                   version='5.1-preview.1',
                   route_values=route_values)

    def get_test_plan_by_id(self, project, plan_id):
        """GetTestPlanById.
        [Preview API] Get a test plan by Id.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan to get.
        :rtype: :class:`<TestPlan> <azure.devops.v5_1.test_plan.models.TestPlan>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        response = self._send(http_method='GET',
                              location_id='0e292477-a0c2-47f3-a9b6-34f153d627f4',
                              version='5.1-preview.1',
                              route_values=route_values)
        return self._deserialize('TestPlan', response)

    def get_test_plans(self, project, owner=None, continuation_token=None, include_plan_details=None, filter_active_plans=None):
        """GetTestPlans.
        [Preview API] Get a list of test plans
        :param str project: Project ID or project name
        :param str owner: Filter for test plan by owner ID or name
        :param str continuation_token: If the list of plans returned is not complete, a continuation token to query next batch of plans is included in the response header as "x-ms-continuationtoken". Omit this parameter to get the first batch of test plans.
        :param bool include_plan_details: Get all properties of the test plan
        :param bool filter_active_plans: Get just the active plans
        :rtype: :class:`<GetTestPlansResponseValue>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        query_parameters = {}
        if owner is not None:
            query_parameters['owner'] = self._serialize.query('owner', owner, 'str')
        if continuation_token is not None:
            query_parameters['continuationToken'] = self._serialize.query('continuation_token', continuation_token, 'str')
        if include_plan_details is not None:
            query_parameters['includePlanDetails'] = self._serialize.query('include_plan_details', include_plan_details, 'bool')
        if filter_active_plans is not None:
            query_parameters['filterActivePlans'] = self._serialize.query('filter_active_plans', filter_active_plans, 'bool')
        response = self._send(http_method='GET',
                              location_id='0e292477-a0c2-47f3-a9b6-34f153d627f4',
                              version='5.1-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        response_value = self._deserialize('[TestPlan]', self._unwrap_collection(response))
        continuation_token = self._get_continuation_token(response)
        return self.GetTestPlansResponseValue(response_value, continuation_token)

    class GetTestPlansResponseValue(object):
        def __init__(self, value, continuation_token):
            """
            Response for the get_test_plans method

            :param value:
            :type value: :class:`<[TestPlan]> <azure.devops.v5_1.test_plan.models.[TestPlan]>`
            :param continuation_token: The continuation token to be used to get the next page of results.
            :type continuation_token: str
            """
            self.value = value
            self.continuation_token = continuation_token

    def update_test_plan(self, test_plan_update_params, project, plan_id):
        """UpdateTestPlan.
        [Preview API] Update a test plan.
        :param :class:`<TestPlanUpdateParams> <azure.devops.v5_1.test_plan.models.TestPlanUpdateParams>` test_plan_update_params: A testPlanUpdateParams object.TestPlanUpdateParams
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan to be updated.
        :rtype: :class:`<TestPlan> <azure.devops.v5_1.test_plan.models.TestPlan>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        content = self._serialize.body(test_plan_update_params, 'TestPlanUpdateParams')
        response = self._send(http_method='PATCH',
                              location_id='0e292477-a0c2-47f3-a9b6-34f153d627f4',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('TestPlan', response)

    def get_suite_entries(self, project, suite_id, suite_entry_type=None):
        """GetSuiteEntries.
        [Preview API] Get a list of test suite entries in the test suite.
        :param str project: Project ID or project name
        :param int suite_id: Id of the parent suite.
        :param str suite_entry_type:
        :rtype: [SuiteEntry]
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        query_parameters = {}
        if suite_entry_type is not None:
            query_parameters['suiteEntryType'] = self._serialize.query('suite_entry_type', suite_entry_type, 'str')
        response = self._send(http_method='GET',
                              location_id='d6733edf-72f1-4252-925b-c560dfe9b75a',
                              version='5.1-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('[SuiteEntry]', self._unwrap_collection(response))

    def reorder_suite_entries(self, suite_entries, project, suite_id):
        """ReorderSuiteEntries.
        [Preview API] Reorder test suite entries in the test suite.
        :param [SuiteEntryUpdateParams] suite_entries: List of SuiteEntry to reorder.
        :param str project: Project ID or project name
        :param int suite_id: Id of the parent test suite.
        :rtype: [SuiteEntry]
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        content = self._serialize.body(suite_entries, '[SuiteEntryUpdateParams]')
        response = self._send(http_method='PATCH',
                              location_id='d6733edf-72f1-4252-925b-c560dfe9b75a',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('[SuiteEntry]', self._unwrap_collection(response))

    def create_test_suite(self, test_suite_create_params, project, plan_id):
        """CreateTestSuite.
        [Preview API] Create test suite.
        :param :class:`<TestSuiteCreateParams> <azure.devops.v5_1.test_plan.models.TestSuiteCreateParams>` test_suite_create_params: Parameters for suite creation
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan that contains the suites.
        :rtype: :class:`<TestSuite> <azure.devops.v5_1.test_plan.models.TestSuite>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        content = self._serialize.body(test_suite_create_params, 'TestSuiteCreateParams')
        response = self._send(http_method='POST',
                              location_id='1046d5d3-ab61-4ca7-a65a-36118a978256',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('TestSuite', response)

    def delete_test_suite(self, project, plan_id, suite_id):
        """DeleteTestSuite.
        [Preview API] Delete test suite.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan that contains the suite.
        :param int suite_id: ID of the test suite to delete.
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        self._send(http_method='DELETE',
                   location_id='1046d5d3-ab61-4ca7-a65a-36118a978256',
                   version='5.1-preview.1',
                   route_values=route_values)

    def get_test_suite_by_id(self, project, plan_id, suite_id, expand=None):
        """GetTestSuiteById.
        [Preview API] Get test suite by suite id.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan that contains the suites.
        :param int suite_id: ID of the suite to get.
        :param str expand: Include the children suites and testers details
        :rtype: :class:`<TestSuite> <azure.devops.v5_1.test_plan.models.TestSuite>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        query_parameters = {}
        if expand is not None:
            query_parameters['expand'] = self._serialize.query('expand', expand, 'str')
        response = self._send(http_method='GET',
                              location_id='1046d5d3-ab61-4ca7-a65a-36118a978256',
                              version='5.1-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('TestSuite', response)

    def get_test_suites_for_plan(self, project, plan_id, expand=None, continuation_token=None, as_tree_view=None):
        """GetTestSuitesForPlan.
        [Preview API] Get test suites for plan.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan for which suites are requested.
        :param str expand: Include the children suites and testers details.
        :param str continuation_token: If the list of suites returned is not complete, a continuation token to query next batch of suites is included in the response header as "x-ms-continuationtoken". Omit this parameter to get the first batch of test suites.
        :param bool as_tree_view: If the suites returned should be in a tree structure.
        :rtype: :class:`<GetTestSuitesForPlanResponseValue>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        query_parameters = {}
        if expand is not None:
            query_parameters['expand'] = self._serialize.query('expand', expand, 'str')
        if continuation_token is not None:
            query_parameters['continuationToken'] = self._serialize.query('continuation_token', continuation_token, 'str')
        if as_tree_view is not None:
            query_parameters['asTreeView'] = self._serialize.query('as_tree_view', as_tree_view, 'bool')
        response = self._send(http_method='GET',
                              location_id='1046d5d3-ab61-4ca7-a65a-36118a978256',
                              version='5.1-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        response_value = self._deserialize('[TestSuite]', self._unwrap_collection(response))
        continuation_token = self._get_continuation_token(response)
        return self.GetTestSuitesForPlanResponseValue(response_value, continuation_token)

    class GetTestSuitesForPlanResponseValue(object):
        def __init__(self, value, continuation_token):
            """
            Response for the get_test_suites_for_plan method

            :param value:
            :type value: :class:`<[TestSuite]> <azure.devops.v5_1.test_plan.models.[TestSuite]>`
            :param continuation_token: The continuation token to be used to get the next page of results.
            :type continuation_token: str
            """
            self.value = value
            self.continuation_token = continuation_token

    def update_test_suite(self, test_suite_update_params, project, plan_id, suite_id):
        """UpdateTestSuite.
        [Preview API] Update test suite.
        :param :class:`<TestSuiteUpdateParams> <azure.devops.v5_1.test_plan.models.TestSuiteUpdateParams>` test_suite_update_params: Parameters for suite updation
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan that contains the suites.
        :param int suite_id: ID of the parent suite.
        :rtype: :class:`<TestSuite> <azure.devops.v5_1.test_plan.models.TestSuite>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        content = self._serialize.body(test_suite_update_params, 'TestSuiteUpdateParams')
        response = self._send(http_method='PATCH',
                              location_id='1046d5d3-ab61-4ca7-a65a-36118a978256',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('TestSuite', response)

    def get_suites_by_test_case_id(self, test_case_id):
        """GetSuitesByTestCaseId.
        Find the list of all test suites in which a given test case is present. This is helpful if you need to find out which test suites are using a test case, when you need to make changes to a test case.
        :param int test_case_id: ID of the test case for which suites need to be fetched.
        :rtype: [TestSuite]
        """
        query_parameters = {}
        if test_case_id is not None:
            query_parameters['testCaseId'] = self._serialize.query('test_case_id', test_case_id, 'int')
        response = self._send(http_method='GET',
                              location_id='a4080e84-f17b-4fad-84f1-7960b6525bf2',
                              version='5.1',
                              query_parameters=query_parameters)
        return self._deserialize('[TestSuite]', self._unwrap_collection(response))

    def add_test_cases_to_suite(self, suite_test_case_create_update_parameters, project, plan_id, suite_id):
        """AddTestCasesToSuite.
        [Preview API] Add test cases to a suite with specified configurations
        :param [SuiteTestCaseCreateUpdateParameters] suite_test_case_create_update_parameters: SuiteTestCaseCreateUpdateParameters object.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan to which test cases are to be added.
        :param int suite_id: ID of the test suite to which test cases are to be added.
        :rtype: [TestCase]
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        content = self._serialize.body(suite_test_case_create_update_parameters, '[SuiteTestCaseCreateUpdateParameters]')
        response = self._send(http_method='POST',
                              location_id='a9bd61ac-45cf-4d13-9441-43dcd01edf8d',
                              version='5.1-preview.2',
                              route_values=route_values,
                              content=content)
        return self._deserialize('[TestCase]', self._unwrap_collection(response))

    def get_test_case(self, project, plan_id, suite_id, test_case_ids, wit_fields=None, return_identity_ref=None):
        """GetTestCase.
        [Preview API] Get Test Cases For a Suite.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan for which test cases are requested.
        :param int suite_id: ID of the test suite for which test cases are requested.
        :param str test_case_ids: Test Case Ids to be fetched.
        :param str wit_fields: Get the list of witFields.
        :param bool return_identity_ref: If set to true, returns all identity fields, like AssignedTo, ActivatedBy etc., as IdentityRef objects. If set to false, these fields are returned as unique names in string format. This is false by default.
        :rtype: [TestCase]
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        if test_case_ids is not None:
            route_values['testCaseIds'] = self._serialize.url('test_case_ids', test_case_ids, 'str')
        query_parameters = {}
        if wit_fields is not None:
            query_parameters['witFields'] = self._serialize.query('wit_fields', wit_fields, 'str')
        if return_identity_ref is not None:
            query_parameters['returnIdentityRef'] = self._serialize.query('return_identity_ref', return_identity_ref, 'bool')
        response = self._send(http_method='GET',
                              location_id='a9bd61ac-45cf-4d13-9441-43dcd01edf8d',
                              version='5.1-preview.2',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('[TestCase]', self._unwrap_collection(response))

    def get_test_case_list(self, project, plan_id, suite_id, test_ids=None, configuration_ids=None, wit_fields=None, continuation_token=None, return_identity_ref=None, expand=None):
        """GetTestCaseList.
        [Preview API] Get Test Case List return those test cases which have all the configuration Ids as mentioned in the optional paramter. If configuration Ids is null, it return all the test cases
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan for which test cases are requested.
        :param int suite_id: ID of the test suite for which test cases are requested.
        :param str test_ids: Test Case Ids to be fetched.
        :param str configuration_ids: Fetch Test Cases which contains all the configuration Ids specified.
        :param str wit_fields: Get the list of witFields.
        :param str continuation_token: If the list of test cases returned is not complete, a continuation token to query next batch of test cases is included in the response header as "x-ms-continuationtoken". Omit this parameter to get the first batch of test cases.
        :param bool return_identity_ref: If set to true, returns all identity fields, like AssignedTo, ActivatedBy etc., as IdentityRef objects. If set to false, these fields are returned as unique names in string format. This is false by default.
        :param bool expand: If set to false, will get a smaller payload containing only basic details about the suite test case object
        :rtype: :class:`<GetTestCaseListResponseValue>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        query_parameters = {}
        if test_ids is not None:
            query_parameters['testIds'] = self._serialize.query('test_ids', test_ids, 'str')
        if configuration_ids is not None:
            query_parameters['configurationIds'] = self._serialize.query('configuration_ids', configuration_ids, 'str')
        if wit_fields is not None:
            query_parameters['witFields'] = self._serialize.query('wit_fields', wit_fields, 'str')
        if continuation_token is not None:
            query_parameters['continuationToken'] = self._serialize.query('continuation_token', continuation_token, 'str')
        if return_identity_ref is not None:
            query_parameters['returnIdentityRef'] = self._serialize.query('return_identity_ref', return_identity_ref, 'bool')
        if expand is not None:
            query_parameters['expand'] = self._serialize.query('expand', expand, 'bool')
        response = self._send(http_method='GET',
                              location_id='a9bd61ac-45cf-4d13-9441-43dcd01edf8d',
                              version='5.1-preview.2',
                              route_values=route_values,
                              query_parameters=query_parameters)
        response_value = self._deserialize('[TestCase]', self._unwrap_collection(response))
        continuation_token = self._get_continuation_token(response)
        return self.GetTestCaseListResponseValue(response_value, continuation_token)

    class GetTestCaseListResponseValue(object):
        def __init__(self, value, continuation_token):
            """
            Response for the get_test_case_list method

            :param value:
            :type value: :class:`<[TestCase]> <azure.devops.v5_1.test_plan.models.[TestCase]>`
            :param continuation_token: The continuation token to be used to get the next page of results.
            :type continuation_token: str
            """
            self.value = value
            self.continuation_token = continuation_token

    def remove_test_cases_from_suite(self, project, plan_id, suite_id, test_case_ids):
        """RemoveTestCasesFromSuite.
        [Preview API] Removes test cases from a suite based on the list of test case Ids provided.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan from which test cases are to be removed.
        :param int suite_id: ID of the test suite from which test cases are to be removed.
        :param str test_case_ids: Test Case Ids to be removed.
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        if test_case_ids is not None:
            route_values['testCaseIds'] = self._serialize.url('test_case_ids', test_case_ids, 'str')
        self._send(http_method='DELETE',
                   location_id='a9bd61ac-45cf-4d13-9441-43dcd01edf8d',
                   version='5.1-preview.2',
                   route_values=route_values)

    def update_suite_test_cases(self, suite_test_case_create_update_parameters, project, plan_id, suite_id):
        """UpdateSuiteTestCases.
        [Preview API] Update the configurations for test cases
        :param [SuiteTestCaseCreateUpdateParameters] suite_test_case_create_update_parameters: A SuiteTestCaseCreateUpdateParameters object.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan to which test cases are to be updated.
        :param int suite_id: ID of the test suite to which test cases are to be updated.
        :rtype: [TestCase]
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        content = self._serialize.body(suite_test_case_create_update_parameters, '[SuiteTestCaseCreateUpdateParameters]')
        response = self._send(http_method='PATCH',
                              location_id='a9bd61ac-45cf-4d13-9441-43dcd01edf8d',
                              version='5.1-preview.2',
                              route_values=route_values,
                              content=content)
        return self._deserialize('[TestCase]', self._unwrap_collection(response))

    def delete_test_case(self, project, test_case_id):
        """DeleteTestCase.
        [Preview API] Delete a test case.
        :param str project: Project ID or project name
        :param int test_case_id: Id of test case to be deleted.
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if test_case_id is not None:
            route_values['testCaseId'] = self._serialize.url('test_case_id', test_case_id, 'int')
        self._send(http_method='DELETE',
                   location_id='29006fb5-816b-4ff7-a329-599943569229',
                   version='5.1-preview.1',
                   route_values=route_values)

    def clone_test_plan(self, clone_request_body, project, deep_clone=None):
        """CloneTestPlan.
        [Preview API] Clone test plan
        :param :class:`<CloneTestPlanParams> <azure.devops.v5_1.test_plan.models.CloneTestPlanParams>` clone_request_body: Plan Clone Request Body detail TestPlanCloneRequest
        :param str project: Project ID or project name
        :param bool deep_clone: Clones all the associated test cases as well
        :rtype: :class:`<CloneTestPlanOperationInformation> <azure.devops.v5_1.test_plan.models.CloneTestPlanOperationInformation>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        query_parameters = {}
        if deep_clone is not None:
            query_parameters['deepClone'] = self._serialize.query('deep_clone', deep_clone, 'bool')
        content = self._serialize.body(clone_request_body, 'CloneTestPlanParams')
        response = self._send(http_method='POST',
                              location_id='e65df662-d8a3-46c7-ae1c-14e2d4df57e1',
                              version='5.1-preview.2',
                              route_values=route_values,
                              query_parameters=query_parameters,
                              content=content)
        return self._deserialize('CloneTestPlanOperationInformation', response)

    def get_clone_information(self, project, clone_operation_id):
        """GetCloneInformation.
        [Preview API] Get clone information.
        :param str project: Project ID or project name
        :param int clone_operation_id: Operation ID returned when we queue a clone operation
        :rtype: :class:`<CloneTestPlanOperationInformation> <azure.devops.v5_1.test_plan.models.CloneTestPlanOperationInformation>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if clone_operation_id is not None:
            route_values['cloneOperationId'] = self._serialize.url('clone_operation_id', clone_operation_id, 'int')
        response = self._send(http_method='GET',
                              location_id='e65df662-d8a3-46c7-ae1c-14e2d4df57e1',
                              version='5.1-preview.2',
                              route_values=route_values)
        return self._deserialize('CloneTestPlanOperationInformation', response)

    def get_points(self, project, plan_id, suite_id, point_ids, return_identity_ref=None):
        """GetPoints.
        [Preview API] Get a list of points based on point Ids provided.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan for which test points are requested.
        :param int suite_id: ID of the test suite for which test points are requested.
        :param str point_ids: ID of test points to be fetched.
        :param bool return_identity_ref: If set to true, returns the AssignedTo field in TestCaseReference as IdentityRef object.
        :rtype: [TestPoint]
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        if point_ids is not None:
            route_values['pointIds'] = self._serialize.url('point_ids', point_ids, 'str')
        query_parameters = {}
        if return_identity_ref is not None:
            query_parameters['returnIdentityRef'] = self._serialize.query('return_identity_ref', return_identity_ref, 'bool')
        response = self._send(http_method='GET',
                              location_id='52df686e-bae4-4334-b0ee-b6cf4e6f6b73',
                              version='5.1-preview.2',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('[TestPoint]', self._unwrap_collection(response))

    def get_points_list(self, project, plan_id, suite_id, test_point_ids=None, test_case_id=None, continuation_token=None, return_identity_ref=None, include_point_details=None):
        """GetPointsList.
        [Preview API] Get all the points inside a suite based on some filters
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan for which test points are requested.
        :param int suite_id: ID of the test suite for which test points are requested
        :param str test_point_ids: ID of test points to fetch.
        :param str test_case_id: Get Test Points for specific test case Ids.
        :param str continuation_token: If the list of test point returned is not complete, a continuation token to query next batch of test points is included in the response header as "x-ms-continuationtoken". Omit this parameter to get the first batch of test points.
        :param bool return_identity_ref: If set to true, returns the AssignedTo field in TestCaseReference as IdentityRef object.
        :param bool include_point_details: If set to false, returns only necessary information
        :rtype: :class:`<GetPointsListResponseValue>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        query_parameters = {}
        if test_point_ids is not None:
            query_parameters['testPointIds'] = self._serialize.query('test_point_ids', test_point_ids, 'str')
        if test_case_id is not None:
            query_parameters['testCaseId'] = self._serialize.query('test_case_id', test_case_id, 'str')
        if continuation_token is not None:
            query_parameters['continuationToken'] = self._serialize.query('continuation_token', continuation_token, 'str')
        if return_identity_ref is not None:
            query_parameters['returnIdentityRef'] = self._serialize.query('return_identity_ref', return_identity_ref, 'bool')
        if include_point_details is not None:
            query_parameters['includePointDetails'] = self._serialize.query('include_point_details', include_point_details, 'bool')
        response = self._send(http_method='GET',
                              location_id='52df686e-bae4-4334-b0ee-b6cf4e6f6b73',
                              version='5.1-preview.2',
                              route_values=route_values,
                              query_parameters=query_parameters)
        response_value = self._deserialize('[TestPoint]', self._unwrap_collection(response))
        continuation_token = self._get_continuation_token(response)
        return self.GetPointsListResponseValue(response_value, continuation_token)

    class GetPointsListResponseValue(object):
        def __init__(self, value, continuation_token):
            """
            Response for the get_points_list method

            :param value:
            :type value: :class:`<[TestPoint]> <azure.devops.v5_1.test_plan.models.[TestPoint]>`
            :param continuation_token: The continuation token to be used to get the next page of results.
            :type continuation_token: str
            """
            self.value = value
            self.continuation_token = continuation_token

    def update_test_points(self, test_point_update_params, project, plan_id, suite_id):
        """UpdateTestPoints.
        [Preview API] Update Test Points. This is used to Reset test point to active, update the outcome of a test point or update the tester of a test point
        :param [TestPointUpdateParams] test_point_update_params: A TestPointUpdateParams Object.
        :param str project: Project ID or project name
        :param int plan_id: ID of the test plan for which test points are requested.
        :param int suite_id: ID of the test suite for which test points are requested.
        :rtype: [TestPoint]
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if plan_id is not None:
            route_values['planId'] = self._serialize.url('plan_id', plan_id, 'int')
        if suite_id is not None:
            route_values['suiteId'] = self._serialize.url('suite_id', suite_id, 'int')
        content = self._serialize.body(test_point_update_params, '[TestPointUpdateParams]')
        response = self._send(http_method='PATCH',
                              location_id='52df686e-bae4-4334-b0ee-b6cf4e6f6b73',
                              version='5.1-preview.2',
                              route_values=route_values,
                              content=content)
        return self._deserialize('[TestPoint]', self._unwrap_collection(response))

    def clone_test_suite(self, clone_request_body, project, deep_clone=None):
        """CloneTestSuite.
        [Preview API] Clone test suite
        :param :class:`<CloneTestSuiteParams> <azure.devops.v5_1.test_plan.models.CloneTestSuiteParams>` clone_request_body: Suite Clone Request Body detail TestSuiteCloneRequest
        :param str project: Project ID or project name
        :param bool deep_clone: Clones all the associated test cases as well
        :rtype: :class:`<CloneTestSuiteOperationInformation> <azure.devops.v5_1.test_plan.models.CloneTestSuiteOperationInformation>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        query_parameters = {}
        if deep_clone is not None:
            query_parameters['deepClone'] = self._serialize.query('deep_clone', deep_clone, 'bool')
        content = self._serialize.body(clone_request_body, 'CloneTestSuiteParams')
        response = self._send(http_method='POST',
                              location_id='181d4c97-0e98-4ee2-ad6a-4cada675e555',
                              version='5.1-preview.2',
                              route_values=route_values,
                              query_parameters=query_parameters,
                              content=content)
        return self._deserialize('CloneTestSuiteOperationInformation', response)

    def get_suite_clone_information(self, project, clone_operation_id):
        """GetSuiteCloneInformation.
        [Preview API] Get clone information.
        :param str project: Project ID or project name
        :param int clone_operation_id: Operation ID returned when we queue a clone operation
        :rtype: :class:`<CloneTestSuiteOperationInformation> <azure.devops.v5_1.test_plan.models.CloneTestSuiteOperationInformation>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if clone_operation_id is not None:
            route_values['cloneOperationId'] = self._serialize.url('clone_operation_id', clone_operation_id, 'int')
        response = self._send(http_method='GET',
                              location_id='181d4c97-0e98-4ee2-ad6a-4cada675e555',
                              version='5.1-preview.2',
                              route_values=route_values)
        return self._deserialize('CloneTestSuiteOperationInformation', response)

    def create_test_variable(self, test_variable_create_update_parameters, project):
        """CreateTestVariable.
        [Preview API] Create a test variable.
        :param :class:`<TestVariableCreateUpdateParameters> <azure.devops.v5_1.test_plan.models.TestVariableCreateUpdateParameters>` test_variable_create_update_parameters: TestVariableCreateUpdateParameters
        :param str project: Project ID or project name
        :rtype: :class:`<TestVariable> <azure.devops.v5_1.test_plan.models.TestVariable>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        content = self._serialize.body(test_variable_create_update_parameters, 'TestVariableCreateUpdateParameters')
        response = self._send(http_method='POST',
                              location_id='2c61fac6-ac4e-45a5-8c38-1c2b8fd8ea6c',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('TestVariable', response)

    def delete_test_variable(self, project, test_variable_id):
        """DeleteTestVariable.
        [Preview API] Delete a test variable by its ID.
        :param str project: Project ID or project name
        :param int test_variable_id: ID of the test variable to delete.
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if test_variable_id is not None:
            route_values['testVariableId'] = self._serialize.url('test_variable_id', test_variable_id, 'int')
        self._send(http_method='DELETE',
                   location_id='2c61fac6-ac4e-45a5-8c38-1c2b8fd8ea6c',
                   version='5.1-preview.1',
                   route_values=route_values)

    def get_test_variable_by_id(self, project, test_variable_id):
        """GetTestVariableById.
        [Preview API] Get a test variable by its ID.
        :param str project: Project ID or project name
        :param int test_variable_id: ID of the test variable to get.
        :rtype: :class:`<TestVariable> <azure.devops.v5_1.test_plan.models.TestVariable>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if test_variable_id is not None:
            route_values['testVariableId'] = self._serialize.url('test_variable_id', test_variable_id, 'int')
        response = self._send(http_method='GET',
                              location_id='2c61fac6-ac4e-45a5-8c38-1c2b8fd8ea6c',
                              version='5.1-preview.1',
                              route_values=route_values)
        return self._deserialize('TestVariable', response)

    def get_test_variables(self, project, continuation_token=None):
        """GetTestVariables.
        [Preview API] Get a list of test variables.
        :param str project: Project ID or project name
        :param str continuation_token: If the list of variables returned is not complete, a continuation token to query next batch of variables is included in the response header as "x-ms-continuationtoken". Omit this parameter to get the first batch of test variables.
        :rtype: :class:`<GetTestVariablesResponseValue>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        query_parameters = {}
        if continuation_token is not None:
            query_parameters['continuationToken'] = self._serialize.query('continuation_token', continuation_token, 'str')
        response = self._send(http_method='GET',
                              location_id='2c61fac6-ac4e-45a5-8c38-1c2b8fd8ea6c',
                              version='5.1-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        response_value = self._deserialize('[TestVariable]', self._unwrap_collection(response))
        continuation_token = self._get_continuation_token(response)
        return self.GetTestVariablesResponseValue(response_value, continuation_token)

    class GetTestVariablesResponseValue(object):
        def __init__(self, value, continuation_token):
            """
            Response for the get_test_variables method

            :param value:
            :type value: :class:`<[TestVariable]> <azure.devops.v5_1.test_plan.models.[TestVariable]>`
            :param continuation_token: The continuation token to be used to get the next page of results.
            :type continuation_token: str
            """
            self.value = value
            self.continuation_token = continuation_token

    def update_test_variable(self, test_variable_create_update_parameters, project, test_variable_id):
        """UpdateTestVariable.
        [Preview API] Update a test variable by its ID.
        :param :class:`<TestVariableCreateUpdateParameters> <azure.devops.v5_1.test_plan.models.TestVariableCreateUpdateParameters>` test_variable_create_update_parameters: TestVariableCreateUpdateParameters
        :param str project: Project ID or project name
        :param int test_variable_id: ID of the test variable to update.
        :rtype: :class:`<TestVariable> <azure.devops.v5_1.test_plan.models.TestVariable>`
        """
        route_values = {}
        if project is not None:
            route_values['project'] = self._serialize.url('project', project, 'str')
        if test_variable_id is not None:
            route_values['testVariableId'] = self._serialize.url('test_variable_id', test_variable_id, 'int')
        content = self._serialize.body(test_variable_create_update_parameters, 'TestVariableCreateUpdateParameters')
        response = self._send(http_method='PATCH',
                              location_id='2c61fac6-ac4e-45a5-8c38-1c2b8fd8ea6c',
                              version='5.1-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('TestVariable', response)

